// ESAF : Euso Simulation and Analysis Framework
// $Id: EEventViewer.C,v 1.56 2006/02/07 09:56:38 moreggia Exp $
// Authors: A.Thea, D.Naumov   Aug,  5 2004

//______________________________________________________________________________
//
// EventViewer macro:
// 
// EventViewer is an interface for the classes of eventviewer library made to 
// display a single event or a part of it (the shower, the detector, etc.)
// It doesn't pretend to be "The ESAF Event Display" but just a working and 
// hopefully useful example to show how eventviewer classes work.
//
// See also: TreeViewer macro.
// 


#include "Riostream.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TTree.h"
#include "TControlBar.h"
#include "TSystem.h"
#include "TGFileDialog.h"
#include "TPaveText.h"
#include "TString.h"
#include "TView.h"

#include "EShowerPainter.hh"
#include "EShowerHistoPainter.hh"
#include "EAtmosphereHistoPainter.hh"
#include "ESinglePhotonsPainter.hh"
#include "EFocalSurfacePainter.hh"
#include "ETriggerPainter.hh"
#include "EDetectorHistoPainter.hh"
#include "EDetectorPhotonsPainter.hh"
#include "EOpticsPhotonsPainter.hh"
#include "EEvent.hh"
#include "ERunParameters.hh"
#include "ETruth.hh"


Bool_t animate = kFALSE;
Bool_t axis = kTRUE;

const Double_t km = 1.e6;
const Double_t cm = 10.;
const Double_t cm2 = cm*cm;
const Double_t mm = 1.;
const Double_t m = 1.e3;
const Double_t joule = 1.e-6/(1.60217733e-19);
const Double_t s = 1.e9;
const Double_t g = 1.e-3*joule*s*s/(m*m);


// define in case only atmosphere has been simulated
Double_t GTUlength = 2500;  // in ns  (~500ns for ground detector)  //GRNDetec
Double_t DetectorRadius = 1250*mm;
TVector3 DetectorPos(0,0,430*km);


EGViewer* gPad_shower = 0;
EGViewer* gPad_atmo_photons = 0;
EGViewer* gPad_transmission = 0;
EGViewer* gPad_all_p_on_pupil = 0;
EGViewer* gPad_detail_p_on_pupil = 0;

EShowerPainter*            gShowerPainter = 0;
EShowerHistoPainter*       gShowerHistoPainter = 0;
EAtmosphereHistoPainter*   gAtmosphereHistoPainter = 0;
ESinglePhotonsPainter*     gSinglePhotonsPainter = 0;
EFocalSurfacePainter*      gFocalSurfacePainter = 0;
ETriggerPainter*           gTriggerPainter = 0;
EDetectorHistoPainter*     gFocalSurfaceHistoPainter = 0;
EDetectorPhotonsPainter*   gDetectorPhotonsPainter = 0;
EOpticsPhotonsPainter*     gOpticsPhotonsPainter = 0;

TControlBar *bar      = 0;
TControlBar *selector = 0;

Int_t   gEventCounter = -1;

EEvent *ev = 0;
ERunParameters *runpars = 0;
ETruth *gTruth = NULL;

const char *defaultfilename= "";//"output/simu.root";

enum EAllCanvasPadID {
		kShowerPadID            = 1,
		kShowerHistoPadID       = 2,
		kFocalPlanePadID        = 3,
                kFocalPlaneHistos1PadID = 4,
                kTriggerPadID           = 5,
                kTruthPadID             = 6
};


enum EAllCanvasPad2D {
		kAtmoHisto1PadID= 1,
                kAtmoHisto11PadID= 2,
                kAtmoHisto111PadID= 3,
                kAtmoHisto2PadID= 4,
                kAtmoHisto3PadID= 5,
                kAtmoHisto4PadID= 6
};
enum EAllCanvasPad3D {
		kSinglePhotonsPadID= 1,
		kDetectorPhotonsPadID= 2,
                kOpticsPhotonsPadID= 3,
};

//Watts fix interfaces
void init( const char* fname = 0 );
void refresh();
void showerXY(Option_t *opt);
void shower();
void singlecanvas();
void showerhistos();
void focalplane();
void focalplanehistos1();
void focalplanehistos2();
void trigger();
void truth();
void openfile();
void gettree();
Bool_t loadevent( Int_t i=0);
//void atmohistos1();
//void atmohistos11();
//void atmohistos111();
//void atmohistos2();
//void atmohistos3();
//void atmohistos4();
//void singlephotons();
void detectorphotons();
void opticsphotons();
void photonsgeneration();
void photonsgeneration_alt();
void photonsgeneration_longi();
void transmission();
void directphotons();
void scatphotons();
void reflectphotons();

//______________________________________________________________________________
void EEventViewer( const char* fname = 0  )
{
    init( fname );
    bar = new TControlBar("vertical", "Simu Event Viewer",10,10);

    bar->AddButton("How to run        ", "help()","Instructions for running this macro"); 
    bar->AddButton("Select Event      ", "selectevent()","Select an event"); 
    bar->AddButton("Open File         ", "openfile()","Open a new file"); 
    bar->AddButton("Browser           ", "TBrowser b","Open a Browser"); 
    bar->AddButton("Next Event        ", "nextevent()","Load next event"); 
    bar->AddButton("Prev Event        ", "prevevent()","Load previous event"); 
    bar->AddSeparator();
    bar->AddButton("Event Truth       ", "truth()","Show truth infos on event"); 
    if (gShowerPainter) { 
        bar->AddSeparator();
        bar->AddButton("3D Shower View    ", "shower()","Show 3D Shower"); 
        bar->AddButton("XY Shower View    ", "showerXY(\"XY\")","Show XY Shower"); 
        bar->AddButton("XYZ Shower View   ", "showerXY(\"XYZ\")","Show XYZ Shower"); 
        bar->AddButton("Shower Histograms ", "showerhistos()","Show Shower Histos");
    }

    if ( gAtmosphereHistoPainter ) {
        bar->AddSeparator();
        bar->AddButton("Photons Generation", "photonsgeneration()","Show Atmosphere Histos A");
        bar->AddButton("Photons Generation VS altitude", "photonsgeneration_alt()","Show Atmosphere Histos A_alt");
        bar->AddButton("Photons longitudinal profiles", "photonsgeneration_longi()","Show longitudinal profiles");
        bar->AddButton("Transmission","transmission()","Fluo and Cerenkov transmission");	
        bar->AddButton("Direct photons","directphotons()","Direct photons features");
        bar->AddButton("Scattered photons","scatphotons()","Scattered photons features");
        bar->AddButton("Reflected photons","reflectphotons()","Reflected photons features");
        bar->AddButton("All photons", "AllPhotons()","All photons features");
        bar->AddButton("Multiple scattering","mscatt()","Multiple scattering contribution");	
        bar->AddButton("MScatt CUT","mscattCUT()","Multiple scattering contribution");	
    }

    if ( gFocalSurfacePainter || gFocalSurfaceHistoPainter ) 
        bar->AddSeparator();
    if ( gFocalSurfacePainter )
        bar->AddButton("FocalPlane View   ", "focalplane()","Show Shower Histos");

    if ( gFocalSurfaceHistoPainter ) { 
        bar->AddButton("FocalPlane Histos A","focalplanehistos1()","Show Focal Histos A");
        bar->AddButton("FocalPlane Histos B","focalplanehistos2()","Show Focal Histos B");
    }

    if ( gTriggerPainter ) {
        bar->AddSeparator();
        bar->AddButton("Trigger View   ", "trigger()","Show Trigger");
    }

    if ( gDetectorPhotonsPainter ) {
        bar->AddSeparator();
        bar->AddButton("3D Detector View", "detectorphotons()","Show 3D Photons in Detector");
    }

    if ( gOpticsPhotonsPainter ) {
        bar->AddSeparator();
        bar->AddButton("3D Optics View", "opticsphotons()","Show 3D Photons in Optics");
    }

    bar->AddSeparator();
    bar->AddButton("Animation On/Off ",  "animation()","Animate On/Off");
    bar->AddButton("Axes ON/OFF","axes()","Toggle axes ON/OFF");
    bar->AddButton("Single Canvas ON/OFF","singlecanvas()","Toggle single canvas ON/OFF");
    bar->AddSeparator();
    bar->AddButton("Quit Root        ",  ".q","Quit root");
    bar->Show();
    gROOT->SaveContext();
}

//______________________________________________________________________________
void selectevent( Int_t first=0) {
    Int_t maxNumButtons = 15;
    Int_t numButtons;

    if ( selector ) delete selector;
    if ( !selector ) {
        selector = new TControlBar("vertical", "Event Selector",10,10);


        TTree *etree = (TTree*)gDirectory->Get("etree");
        Long64_t nEv = (Long64_t)etree->GetEntries();

        if ((nEv-first)/maxNumButtons > 0)
            numButtons = maxNumButtons;
        else
            numButtons = (nEv-first)%maxNumButtons;
        
        if ( first > 0 ) 
            selector->AddButton("Prev",Form("selectevent(%d)",
                        first-maxNumButtons > 0 ? first-maxNumButtons : 0),"Test not stable");
            
        for( Long64_t i(0); i<numButtons; i++) {
            selector->AddButton(Form("Event %d",i+first),Form("loadevent(%d)",i+first),"Test not stable");
        } 
        
        if ( first+maxNumButtons < nEv )
           selector->AddButton("Next",Form("selectevent(%d)",first+maxNumButtons),"");
        selector->AddButton("Hide","selector->Hide()","Test");
        gROOT->SaveContext();
    } 

    selector->Show();
    refresh();
}

//______________________________________________________________________________
void singlecanvas()
{
    TCanvas *c, *c1;
    if ( (c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all")) ) {
        Bool_t paintShower      = kFALSE;
        Bool_t paintShowerXY    = kFALSE; 
        Bool_t paintShowerXYZ   = kFALSE;  
        Bool_t paintShowerHisto = kFALSE;
        Bool_t paintFocalPlane  = kFALSE;
        Bool_t paintFocalPlaneHisto1 = kFALSE;
        Bool_t paintFocalPlaneHisto2 = kFALSE;
        Bool_t paintTrigger = kFALSE;
        Bool_t paintTruth = kFALSE;

        TPad *dummy;
        if ( (dummy = (TPad*)c->FindObject(Form("c_all_%d",kShowerPadID))) ) {
            paintShower = (dummy->FindObject("fTOP") != 0);
            paintShowerXY = (dummy->GetListOfPrimitives()->FindObject("ShowerXY") != 0); 
            paintShowerXYZ = (dummy->GetListOfPrimitives()->FindObject("ShowerXYZ") != 0); 
        }
        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kShowerHistoPadID))))
            paintShowerHisto = (dummy->FindObject("Pad1") != 0);
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kFocalPlanePadID))))
            paintFocalPlane = (dummy->GetListOfPrimitives()->FindObject("EFocalSurfacePainter") != 0);
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kFocalPlaneHistos1PadID)))){
            paintFocalPlaneHisto1 = (dummy->FindObject("Pad1") != 0);
            paintFocalPlaneHisto2 = (dummy->FindObject("Pad1") != 0);
        }
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kTriggerPadID))))
            paintTrigger = (dummy->GetListOfPrimitives()->FindObject("ETriggerPainter") != 0);
       if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kTruthPadID))))
            paintTruth = (dummy->GetListOfPrimitives()->FindObject("ETruthPainter") != 0);

        delete c;
        if ( paintShowerXY )         showerXY("XY");
        if ( paintShowerXYZ )        showerXY("XYZ"); 
        if ( paintShower )           shower();
        if ( paintShowerHisto )      showerhistos();
        if ( paintFocalPlane )       focalplane();
        if ( paintFocalPlaneHisto1 ) focalplanehistos1();
        if ( paintFocalPlaneHisto2 ) focalplanehistos2();
        if ( paintTrigger )           trigger();
    } 
    else {
        c = new TCanvas("c_all","c_all");
        c->Divide(2,3);
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_shower") )) {
            delete c1;
            shower();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_showerXY") )) {
            delete c1;
            showerXY("XY");
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_showerXYZ") )) {
            delete c1;
            showerXY("XYZ");
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_shower_histo") )) {
            delete c1;
            showerhistos();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane") )) {
            delete c1;
            focalplane();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo1") )) {
            delete c1;
            focalplanehistos1();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo2") )) {
            delete c1;
            focalplanehistos2();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_trigger") )) {
            delete c1;
            trigger();
        }
        if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_truth") )) {
            delete c1;
            truth();
        }
    }	

}

//______________________________________________________________________________
void axes()
{
   axis = !axis;
   if (!gPad) return;
   TView *view = gPad->GetView();
   if (view) {
       view->ShowAxis();
       view->SetParallel();
   }
}   

//______________________________________________________________________________
void help() {
   //
  
   new TCanvas("chelp","Help to run demos",200,10,700,600);

   TPaveText *welcome = new TPaveText(.1,.8,.9,.97);
   welcome->AddText("Welcome to ESAF Event Viewer");
   welcome->SetTextFont(32);
   welcome->SetTextColor(4);
   welcome->SetFillColor(24);
   welcome->Draw();

   TPaveText *hdemo = new TPaveText(.05,.05,.95,.7);
   hdemo->SetTextAlign(12);
   hdemo->SetTextFont(52);
   hdemo->AddText("- Demo for visualization of the shower development in atmosphere");
   hdemo->AddText("  and signal observed by the EUSO detector");
   hdemo->AddText("- Click left mouse button to execute one demo");
   hdemo->AddText("- Click left mouse button on Animation On/Off to start/stop animation");
   hdemo->AddText(" ");
   hdemo->SetAllWith("....","color",2);
   hdemo->SetAllWith("....","font",72);
   hdemo->SetAllWith("....","size",0.03);

   hdemo->Draw();
}

//______________________________________________________________________________
void warning(TString message) {
    //
    //
    //
    
    TCanvas *c=0;
    if ((c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_warning") )) {
      c->cd();
      c->Clear();
    }
    else            
       c = new TCanvas("c_warning","Warning",-1);

   TPaveText *warning = new TPaveText(.1,.8,.9,.97);
   warning->AddText("WARNING! ATTENZIONE! VNIMANIE! ACHTUNG!");
   warning->AddText(message);
   warning->AddText("Please close this window manually.");
   warning->SetTextFont(32);
   warning->SetTextColor(4);
   warning->SetFillColor(24);
   warning->Draw();
   c->Modified();
}

//______________________________________________________________________________
void init( const char* fname) {
    //
    //
    //
    const char* filename =  fname ? fname : defaultfilename;

    TFile *f = new TFile(filename);
    if ( f->IsZombie() ) {
        delete f;
        Printf("Unable to open file %s. Opening browser...",filename);
        openfile();
    }
    else gettree();
}

//______________________________________________________________________________
void gettree() {
    //
    //
    //
    
    TTree *etree = (TTree*)gDirectory->Get("etree");
    if ( etree ) {
        Printf("etree found with %d events stored.", (Int_t)etree->GetEntries());
        loadevent();
    }
}
//______________________________________________________________________________
void openfile() {
    //
    //
    //

    if ( ev ) {
        delete ev;
        ev = 0;
    }

    if ( runpars ) {
        delete runpars;
        runpars = 0;
    }

    if ( selector ) {
        delete selector;
        selector = 0;
    }

    TFile *f = 0;
    TGFileInfo* info = new TGFileInfo();
    new TGFileDialog(gClient->GetRoot(), 0, kFDOpen, info);
    if ( info->fFilename ) 
        f = new TFile( info->fFilename );

    if ( f && !f->IsZombie() ) {
        Printf("%s successfully opened.", info->fFilename);
        gettree();
    } else {
        Printf("Unable to open %s",info->fFilename);
    }
}

//______________________________________________________________________________
Bool_t loadevent( Int_t i) {

    TTree *etree = (TTree*)gDirectory->Get("etree");
        
    if ( !etree ) {
        Printf("init(): unable to retrieve etree from TFile. Exiting");
        return kFALSE;
    }
    
    // associate the tree and the event
    if ( !ev )  { 
        ev = new EEvent();
        ev->SetBranches(etree);
        runpars = ev->GetRunPars();
    }

    cout << Form("Entry %d, %d bytes read", i, etree->GetEntry(i)) << endl;

    // read the value of gtu length
    // if runpars does not exist, one uses GTUlength value defined at present file beginning
    if(runpars && runpars->GetPmtData().GetGtuLength()) GTUlength = runpars->GetPmtData().GetGtuLength();
    //DELETE : remove commentary //GRNDetec

    EAtmosphere *atmosphere = ev->GetAtmosphere();
    EShower *shower = ev->GetShower();
    EDetector *detector = ev->GetDetector();
    EChipTrackTrigger *trigger = ev->GetChipTrackTrigger();

    if ( selector ) selector->Hide();

    gEventCounter = i;

    if (gShowerPainter) delete gShowerPainter;
    if (gShowerHistoPainter) delete gShowerHistoPainter;
    if (gAtmosphereHistoPainter) delete gAtmosphereHistoPainter;
    if (gSinglePhotonsPainter) delete gSinglePhotonsPainter;
    if (gDetectorPhotonsPainter) delete gDetectorPhotonsPainter;
    if (gOpticsPhotonsPainter) delete gOpticsPhotonsPainter;
    if (gFocalSurfacePainter) delete gFocalSurfacePainter;
    if (gFocalSurfaceHistoPainter) delete gFocalSurfaceHistoPainter;
    if (gTriggerPainter) delete gTriggerPainter;
    gTruth = 0; // NOT delete, it belongs to the tree.

    gTruth = ev->GetTruth();
    if(shower) gShowerPainter = new EShowerPainter( shower );
    if(shower) gShowerHistoPainter = new EShowerHistoPainter( shower );
    if(atmosphere) gAtmosphereHistoPainter = new EAtmosphereHistoPainter( atmosphere,GTUlength,DetectorPos,DetectorRadius,gTruth,shower);
    if(atmosphere) gSinglePhotonsPainter = new ESinglePhotonsPainter( atmosphere );
    if(detector) gDetectorPhotonsPainter = new EDetectorPhotonsPainter( detector );
    if(detector) gOpticsPhotonsPainter = new EOpticsPhotonsPainter( detector );
    if(detector) gFocalSurfacePainter = new EFocalSurfacePainter( detector, runpars );
    if(detector) gFocalSurfaceHistoPainter = new EDetectorHistoPainter( detector, runpars );
    if(detector) gTriggerPainter = new ETriggerPainter( detector, runpars, trigger);

    return kTRUE;
    
}

//______________________________________________________________________________
void nextevent() {

    TTree *etree = (TTree*)gDirectory->Get("etree");
    if ( (gEventCounter+1) >= 0 && (gEventCounter+1) < (Int_t)etree->GetEntriesFast() ) {
         if ( loadevent( gEventCounter+1 ) ) refresh();
    }
    else
        Printf("nextevent: Last event reached.");

}
//______________________________________________________________________________
void prevevent() {

    TTree *etree = (TTree*)gDirectory->Get("etree");
    if ( (gEventCounter-1) >= 0 && (gEventCounter-1) < (Int_t)etree->GetEntriesFast() ) {
        if ( loadevent( gEventCounter-1 ) ) refresh();
    }
    else
        Printf("prevevent: First event reached.");

}
//______________________________________________________________________________
void refresh() {
    TCanvas *c;
    if ((c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        Bool_t paintAtmoHisto1 = kFALSE;
        Bool_t paintAtmoHisto11 = kFALSE;
        Bool_t paintAtmoHisto2 = kFALSE;
        Bool_t paintAtmoHisto3 = kFALSE;
        Bool_t paintAtmoHisto4 = kFALSE;
        Bool_t paintShower      = kFALSE;
        Bool_t paintShowerXY    = kFALSE; 
        Bool_t paintShowerXYZ   = kFALSE;  
        Bool_t paintShowerHisto = kFALSE;
        Bool_t paintFocalPlane  = kFALSE;
        Bool_t paintFocalPlaneHisto1 = kFALSE;
        Bool_t paintFocalPlaneHisto2 = kFALSE;
        Bool_t paintTrigger = kFALSE;

        Bool_t paintSinglePhotons = kFALSE;
        Bool_t paintDetectorPhotons = kFALSE;
        Bool_t paintOpticsPhotons = kFALSE;

        Bool_t paintTruth = kFALSE;
        TPad *dummy;
        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kShowerPadID)) )) {
            paintShower = (dummy->FindObject("fFoV") != 0);
            paintShowerXY = (dummy->GetListOfPrimitives()->FindObject("ShowerXY") != 0); 
            paintShowerXYZ = (dummy->GetListOfPrimitives()->FindObject("ShowerXYZ") != 0); 
        }
        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kShowerHistoPadID))))
            paintShowerHisto = (dummy->FindObject("Pad1") != 0);

        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kAtmoHisto1PadID)))){
            paintAtmoHisto1 = (dummy->FindObject("Pad1") != 0);
            paintAtmoHisto11 = (dummy->FindObject("Pad1") != 0);
            paintAtmoHisto2 = (dummy->FindObject("Pad1") != 0);
            paintAtmoHisto3 = (dummy->FindObject("Pad1") != 0);
            paintAtmoHisto4 = (dummy->FindObject("Pad1") != 0);
        }

        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kSinglePhotonsPadID))))
            paintSinglePhotons = (dummy->FindObject("Pad1") != 0);
        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kDetectorPhotonsPadID))))
            paintDetectorPhotons = (dummy->FindObject("Pad1") != 0);
        if (( dummy = (TPad*)c->FindObject(Form("c_all_%d",kOpticsPhotonsPadID))))
            paintOpticsPhotons = (dummy->FindObject("Pad1") != 0);

        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kFocalPlanePadID))))
            paintFocalPlane = (dummy->GetListOfPrimitives()->FindObject("EFocalSurfacePainter") != 0);
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kFocalPlaneHistos1PadID)))){
            paintFocalPlaneHisto1 = (dummy->FindObject("Pad1") != 0);
            paintFocalPlaneHisto2 = (dummy->FindObject("Pad1") != 0);
        }
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kTriggerPadID))))
            paintTrigger = (dummy->GetListOfPrimitives()->FindObject("ETriggerPainter") != 0);
        if ((dummy = (TPad*)c->FindObject(Form("c_all_%d",kTruthPadID))))
            paintTruth = (dummy->GetListOfPrimitives()->FindObject("ETruthPainter") != 0);

        if ( paintShower )           shower();
        if ( paintShowerXY )         showerXY("XY");
        if ( paintShowerXYZ )        showerXY("XYZ"); 
        if ( paintShowerHisto )      showerhistos();
        if ( paintAtmoHisto1 )       photonsgeneration();
        if ( paintAtmoHisto11 )      photonsgeneration_alt();
        if ( paintDetectorPhotons )  detectorphotons();
        if ( paintOpticsPhotons )    opticsphotons();
        if ( paintFocalPlane )       focalplane();
        if ( paintFocalPlaneHisto1 ) focalplanehistos1();
        if ( paintFocalPlaneHisto2 ) focalplanehistos2();
        if ( paintTrigger )          trigger();
        if ( paintTruth )            truth();
    } else { 
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_shower")) shower();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_showerXY")) showerXY("XY");
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_showerXYZ")) showerXY("XYZ");
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_shower_histo")) showerhistos();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_atmo_histo1")) photonsgeneration();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_atmo_histo11")) photonsgeneration_alt();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_atmo_histo111")) photonsgeneration_longi();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_detectorphotons")) detectorphotons();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_detectorphotons")) opticsphotons();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane")) focalplane();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo1")) focalplanehistos1();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo2")) focalplanehistos2();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_trigger")) trigger();
        if ((TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_truth")) truth();

    }
    
}

//______________________________________________________________________________
EGViewer* checkPad(EGViewer* gv, TString s) {
    if(!gv) {
        gv = new EGViewer();
        gv->SetName(s.Data());
        gv->SetTitle(s.Data());
    }
    return gv;
}

//______________________________________________________________________________
void shower() {

    if (!gShowerHistoPainter) return;
    
    gPad_shower = checkPad(gPad_shower,TString("Shower Viewer"));
    
    TCanvas *c;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kShowerPadID);
    } 
    else {
        c = gPad_shower->FindCanvas("c_shower");
        if(!c ) c = gPad_shower->AddTab("c_shower","c_shower");
        c->Clear();
        c->cd();
    }

    gShowerPainter->Draw();
    TView *view = gPad->GetView();
    if (view) {
        view->ShowAxis();
        view->SetParallel();
    }
    c->Modified();
}

//______________________________________________________________________________
void showerXY(Option_t *opt) {
    
    if (!gShowerHistoPainter) return;
    
    gPad_shower = checkPad(gPad_shower,TString("Shower Viewer"));
    
    TString option(opt);
    TCanvas *c;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kShowerPadID);
    } 
    else {
        if ( option =="XY" ) {
            c = gPad_shower->FindCanvas("c_showerXY");
            if(!c ) c = gPad_shower->AddTab("c_showerXY","c_showerXY");
        }
        else if (option == "XYZ") {
            c = gPad_shower->FindCanvas("c_showerXYZ");
            if(!c ) c = gPad_shower->AddTab("c_showerXYZ","c_showerXYZ");
        }     
        c->Clear();
        c->cd();
    }

    if (option =="XY" ) {
        c->SetRightMargin(0.15);
        gShowerPainter->DrawXY();
//        refresh();
    } else if(option == "XYZ") {
        gShowerPainter->DrawXYZ();    
//        refresh();
    }    
}

//______________________________________________________________________________
void showerhistos() {
    
    if (!gShowerHistoPainter) return;

    gPad_shower = checkPad(gPad_shower,TString("Shower Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kShowerHistoPadID);
    } 
    else {
        c = gPad_shower->FindCanvas("c_shower_histo");
        if(!c ) c = gPad_shower->AddTab("c_shower_histo","c_shower_histo");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.67,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.67,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.34,0.49,0.66);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.34,0.99,0.66);
    TPad *pad5 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.33);
    TPad *pad6 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.33);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    pad5->Draw();
    pad6->Draw();

    pad1->cd();
    gShowerHistoPainter->Draw("Ne");    
    pad2->cd();
    gShowerHistoPainter->Draw("Age");
    pad3->cd();
    gShowerHistoPainter->Draw("Depth");
    pad4->cd();
    gShowerHistoPainter->Draw("Truth");   
    pad5->cd();
    gShowerHistoPainter->Draw("Ne_alt");    
    
}

//______________________________________________________________________________
void photonsgeneration() {
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_atmo_photons = checkPad(gPad_atmo_photons,TString("Atmo photons Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kAtmoHisto1PadID);
    } 
    else {
        c = gPad_atmo_photons->FindCanvas("c_atmo_histo1");
        if(!c ) c = gPad_atmo_photons->AddTab("c_atmo_histo1","c_atmo_histo1");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.67,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.67,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.34,0.49,0.66);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.34,0.99,0.66);
    TPad *pad5 = new TPad("Pad5","Pad5",0.01,0.01,0.49,0.33);
    TPad *pad6 = new TPad("Pad6","Pad6",0.51,0.01,0.99,0.33);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();
    pad5->Draw();
    pad6->Draw();

    pad1->cd();
    gAtmosphereHistoPainter->Draw("Nph_F");
    pad2->cd();
    gAtmosphereHistoPainter->Draw("Nph_C");
    pad3->cd();
    gAtmosphereHistoPainter->Draw("Yield_F");
    pad4->cd();
    gAtmosphereHistoPainter->Draw("Yield_C");
    pad5->cd();
    gAtmosphereHistoPainter->Draw("Sp_F");
    pad6->cd();
    gAtmosphereHistoPainter->Draw("Sp_C");

}

//______________________________________________________________________________
void photonsgeneration_alt() {
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_atmo_photons = checkPad(gPad_atmo_photons,TString("Atmo photons Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kAtmoHisto11PadID);
    } 
    else {
        c = gPad_atmo_photons->FindCanvas("c_atmo_histo11");
        if(!c ) c = gPad_atmo_photons->AddTab("c_atmo_histo11","c_atmo_histo11");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    gAtmosphereHistoPainter->Draw("alt_Nph_F");
    pad2->cd();
    gAtmosphereHistoPainter->Draw("alt_Nph_C");
    pad3->cd();
    gAtmosphereHistoPainter->Draw("alt_Yield_F");
    pad4->cd();
    gAtmosphereHistoPainter->Draw("alt_Yield_C");
}

//______________________________________________________________________________
void photonsgeneration_longi() {
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_atmo_photons = checkPad(gPad_atmo_photons,TString("Atmo photons Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kAtmoHisto111PadID);
    } 
    else {
        c = gPad_atmo_photons->FindCanvas("c_atmo_histo111");
        if(!c ) c = gPad_atmo_photons->AddTab("c_atmo_histo111","c_atmo_histo111");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.51,0.49,0.99);
    TPad *pad2 = new TPad("Pad2","Pad2",0.51,0.51,0.99,0.99);
    TPad *pad3 = new TPad("Pad3","Pad3",0.01,0.01,0.49,0.49);
    TPad *pad4 = new TPad("Pad4","Pad4",0.51,0.01,0.99,0.49);

    pad1->Draw();
    pad2->Draw();
    pad3->Draw();
    pad4->Draw();

    pad1->cd();
    gAtmosphereHistoPainter->Draw("Fluo_longi_prof_km");
    pad2->cd();
    gAtmosphereHistoPainter->Draw("Fluo_longi_prof_gram");
    pad3->cd();
    gAtmosphereHistoPainter->Draw("Ckov_longi_prof_km");
    pad4->cd();
    gAtmosphereHistoPainter->Draw("Ckov_longi_prof_gram");
}

//______________________________________________________________________________
void transmission() {
    //
    // both fluo and cerenkov
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_transmission = checkPad(gPad_transmission,TString("Atmo Transmission Viewer"));

    // transmission overview
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) c->cd(kAtmoHisto1PadID);
    else {
        c = gPad_transmission->FindCanvas("c_atmo_trans");
        if(!c ) c = gPad_transmission->AddTab("c_atmo_trans","c_atmo_trans");
        c->Clear();
        c->cd();
    }
    c->Divide(2,2);
    c->cd(1);
    gAtmosphereHistoPainter->Draw("alt_fluo_trans");
    c->cd(2);
    gAtmosphereHistoPainter->Draw("alt_ckov_trans");
    c->cd(3);
    gAtmosphereHistoPainter->Draw("Sps_F");
    c->cd(4);
    gAtmosphereHistoPainter->Draw("Sps_C");
    
    
    
    // transmission details in altitude
    TCanvas *c1 = 0;
    if (( c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) c1->cd(kAtmoHisto1PadID);
    else {
        c1 = gPad_transmission->FindCanvas("c_atmo_trans1");
        if(!c1 ) c1 = gPad_transmission->AddTab("c_atmo_trans1","c_atmo_trans1");
        c1->Clear();
        c1->cd();
    }
    c1->Divide(2,2);
    c1->cd(1);
    gAtmosphereHistoPainter->Draw("alt_rayltrans_details");
    c1->cd(2);
    gAtmosphereHistoPainter->Draw("alt_ozonetrans_details");
    c1->cd(3);
    gAtmosphereHistoPainter->Draw("alt_aerotrans_details");
    c1->cd(4);
    gAtmosphereHistoPainter->Draw("alt_cloudtrans_details");
    
    
    
    // transmission details in wavelength
    TCanvas *c2 = 0;
    if (( c2 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) c2->cd(kAtmoHisto1PadID);
    else {
        c2 = gPad_transmission->FindCanvas("c_atmo_trans2");
        if(!c2 ) c2 = gPad_transmission->AddTab("c_atmo_trans2","c_atmo_trans2");
        c2->Clear();
        c2->cd();
    }
    c2->Divide(2,2);
    c2->cd(1);
    gAtmosphereHistoPainter->Draw("wl_rayltrans_details");
    c2->cd(2);
    gAtmosphereHistoPainter->Draw("wl_ozonetrans_details");
    c2->cd(3);
    gAtmosphereHistoPainter->Draw("wl_aerotrans_details");
    c2->cd(4);
    gAtmosphereHistoPainter->Draw("wl_cloudtrans_details");
    
    
    
    // total transmission details (altitude and wavelength)
    TCanvas *c21 = 0;
    if (( c21 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) c21->cd(kAtmoHisto1PadID);
    else {
        c21 = gPad_transmission->FindCanvas("c_atmo_trans21");
        if(!c21 ) c21 = gPad_transmission->AddTab("c_atmo_trans21","c_atmo_trans21");
        c21->Clear();
        c21->cd();
    }
    c21->Divide(2,1);
    c21->cd(1);
    gAtmosphereHistoPainter->Draw("alt_tottrans_details");
    c21->cd(2);
    gAtmosphereHistoPainter->Draw("wl_tottrans_details");
}

//______________________________________________________________________________
void directphotons() {
    //
    // both fluo and cerenkov
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_detail_p_on_pupil = checkPad(gPad_detail_p_on_pupil,TString("PoPupil Details Viewer"));
    
    // geometry
    TCanvas *c = 0;
    c = gPad_detail_p_on_pupil->FindCanvas("c_atmo_direct");
    if(!c ) c = gPad_detail_p_on_pupil->AddTab("c_atmo_direct","c_atmo_direct");
    c->Clear();
    c->cd();
    c->Divide(2,2);
    c->cd(1);
    gAtmosphereHistoPainter->Draw("alt_direct");
    c->cd(2);
    gAtmosphereHistoPainter->Draw("longi_direct");
    c->cd(3);
    gAtmosphereHistoPainter->Draw("lateral_direct");
    c->cd(4);
    gAtmosphereHistoPainter->Draw("2D_direct");
    
    
    // time on pupil
    TCanvas *c2 = 0;
    c2 = gPad_detail_p_on_pupil->FindCanvas("c_atmo_direct2");
    if(!c2 ) c2 = gPad_detail_p_on_pupil->AddTab("c_atmo_direct2","c_atmo_direct2");
    c2->Clear();
    c2->cd();
    c2->Divide(2,1);
    c2->cd(1);
    gAtmosphereHistoPainter->Draw("direct_on_pupil");
    c2->cd(2);
    gSinglePhotonsPainter->Draw("direct2");    
    
}

//______________________________________________________________________________
void scatphotons() {
    //
    // both fluo and cerenkov
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_detail_p_on_pupil = checkPad(gPad_detail_p_on_pupil,TString("PoPupil Details Viewer"));
    
    // geometry
    TCanvas *c = 0;
    c = gPad_detail_p_on_pupil->FindCanvas("c_atmo_scat");
    if(!c ) c = gPad_detail_p_on_pupil->AddTab("c_atmo_scat","c_atmo_scat");
    c->Clear();
    c->Divide(2,2);
    c->cd(1);
    gAtmosphereHistoPainter->Draw("alt_scat");
    c->cd(2);
    gAtmosphereHistoPainter->Draw("longi_scat");
    c->cd(3);
    gAtmosphereHistoPainter->Draw("lateral_scat");
    c->cd(4);
    gAtmosphereHistoPainter->Draw("2D_scat");
    
    // idem but distinguish scattering medium
    TCanvas *cc = 0;
    cc = gPad_detail_p_on_pupil->FindCanvas("cc_atmo_scat");
    if(!cc ) cc = gPad_detail_p_on_pupil->AddTab("cc_atmo_scat","cc_atmo_scat");
    cc->Clear();
    cc->cd();
    gAtmosphereHistoPainter->Draw("alt_scat_type");
    
    // air scat and aeroscat
    TCanvas *c2 = 0;
    c2 = gPad_detail_p_on_pupil->FindCanvas("c_atmo_scat2");
    if(!c2 ) c2 = gPad_detail_p_on_pupil->AddTab("c_atmo_scat2","c_atmo_scat2");
    c2->Clear();
    c2->Divide(2,2);
    c2->cd(1);
    gAtmosphereHistoPainter->Draw("air_scat_pupil");
    c2->cd(2);
    gAtmosphereHistoPainter->Draw("aero_scat_pupil");
    c2->cd(3);
    gSinglePhotonsPainter->Draw("raylscat2");    
    c2->cd(4);
    gSinglePhotonsPainter->Draw("aeroscat");    
    
    // cloud scat and total scat
    TCanvas *c3 = 0;
    c3 = gPad_detail_p_on_pupil->FindCanvas("c_atmo_scat3");
    if(!c3 ) c3 = gPad_detail_p_on_pupil->AddTab("c_atmo_scat3","c_atmo_scat3");
    c3->Clear();
    c3->Divide(2,2);
    c3->cd(1);
    gAtmosphereHistoPainter->Draw("cloud_scat_pupil");
    c3->cd(2);
    //gPad->SetFillColor(0);
    gAtmosphereHistoPainter->Draw("total_scat_pupil");
    c3->cd(3);
    gSinglePhotonsPainter->Draw("cloudscat2");    
    c3->cd(4);
    gSinglePhotonsPainter->Draw("raylscat2aeroscat2cloudscat2");    
    
    
}

//______________________________________________________________________________
void reflectphotons() {
    //
    // both fluo and cerenkov
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_detail_p_on_pupil = checkPad(gPad_detail_p_on_pupil,TString("PoPupil Details Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) c->cd(kAtmoHisto1PadID);
    else {
        c = gPad_detail_p_on_pupil->FindCanvas("c_atmo_refl");
        if(!c ) c = gPad_detail_p_on_pupil->AddTab("c_atmo_refl","c_atmo_refl");
        c->Clear();
        c->cd();
    }
    c->Divide(2,1);
    c->cd(1);
    gAtmosphereHistoPainter->Draw("reflected_pupil");
    c->cd(2);
    gSinglePhotonsPainter->Draw("reflected2");    
}

//______________________________________________________________________________
void AllPhotons() {
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_all_p_on_pupil = checkPad(gPad_all_p_on_pupil,TString("PoPupil Total Viewer"));
    
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kAtmoHisto4PadID);
    } 
    else {
        c = gPad_all_p_on_pupil->FindCanvas("c_atmo_histo4");
        if(!c ) c = gPad_all_p_on_pupil->AddTab("c_atmo_histo4","c_atmo_histo4");
        c->Clear();
        c->cd();
    }
    
    c->Divide(2,2);
    c->cd(1);
    gAtmosphereHistoPainter->Draw("Nphs_t");
    c->cd(3);
    gAtmosphereHistoPainter->Draw("Nphs_t_stack");
    gStyle->SetNumberContours(6);
    gStyle->SetPalette(1);
    c->cd(2);
    gAtmosphereHistoPainter->Draw("total_Fov_nocut");
    c->cd(4);
    gAtmosphereHistoPainter->Draw("total_Fov");
    
    
    // angular distribution on pupil
    TCanvas *cc = 0;
    if (( cc = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        cc->cd(kAtmoHisto4PadID);
    } 
    else {
        cc = gPad_all_p_on_pupil->FindCanvas("cc_atmo_histo4");
        if(!cc ) cc = gPad_all_p_on_pupil->AddTab("cc_atmo_histo4","cc_atmo_histo4");
        cc->Clear();
        cc->cd();
    }
    
    cc->Divide(2,2);
    cc->cd(1);
    gAtmosphereHistoPainter->Draw("angul_distrib_all");
    cc->cd(2);
    gAtmosphereHistoPainter->Draw("photon_angul_nbinter");
    cc->cd(3);
    gAtmosphereHistoPainter->Draw("2D_direct");
    cc->cd(4);
    gSinglePhotonsPainter->Draw("direct2raylscat2aeroscat2cloudscat2reflected2");    
}

//______________________________________________________________________________
void mscatt() {
    //
    // both fluo and cerenkov
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    gPad_all_p_on_pupil = checkPad(gPad_all_p_on_pupil,TString("PoPupil Total Viewer"));
    
    TCanvas *c = 0;
    c = gPad_all_p_on_pupil->FindCanvas("c_atmo_mscatt");
    if(!c ) c = gPad_all_p_on_pupil->AddTab("c_atmo_mscatt","c_atmo_mscatt");
    c->Clear();
    c->cd();
    c->Divide(2,2);
    
    c->cd(1);
    gAtmosphereHistoPainter->Draw("nbinteractions");
    c->cd(2);
    gAtmosphereHistoPainter->Draw("omegadiff");
    c->cd(3);
    gAtmosphereHistoPainter->Draw("photon_pupil_nbinter");
    c->cd(4);
    gAtmosphereHistoPainter->Draw("lateral_distrib_nbinter");

    TCanvas *cc = 0;
    cc = gPad_all_p_on_pupil->FindCanvas("c_atmo_mscatt_history");
    if(!cc ) cc = gPad_all_p_on_pupil->AddTab("c_atmo_mscatt_history","c_atmo_mscatt_history");
    cc->Clear();
    cc->cd();
    gAtmosphereHistoPainter->Draw("mscatt_history");
}

//______________________________________________________________________________
void mscattCUT() {
    //
    // define a space-time cut to get a more realistic view of MScatt contribution
    //
    
    if (!gAtmosphereHistoPainter) return;
    
    if(gPad_all_p_on_pupil) gPad_all_p_on_pupil->Clear();
    
    // bdf can be configured (linear interpolation as function of position in FoV
    // SNR can be configured -- if SNR <= 0, it is fixed internally at 3 sigmas above bdf fluctuations
    Double_t BDFmin(0.1), BDFmax(0.1), SNR(1);
    gAtmosphereHistoPainter->MScattCut(BDFmin,BDFmax,SNR);
    gSinglePhotonsPainter->DefineMScattCut(gAtmosphereHistoPainter->GetMScattCut());
}

//______________________________________________________________________________
void detectorphotons() {
    
    if (!gDetectorPhotonsPainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kDetectorPhotonsPadID);
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_detectorphotons");
        if (!c) c = new TCanvas("c_detectorphotons","c_detectorphotons");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.01,0.99,0.99);
    pad1->SetFillColor(1);
    pad1->Draw();
    pad1->cd();
    gDetectorPhotonsPainter->Draw();    

}

//______________________________________________________________________________
void opticsphotons() {

    if (!gOpticsPhotonsPainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kOpticsPhotonsPadID);
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_opticsphotons");
        if (!c) c = new TCanvas("c_opticsphotons","c_opticsphotons");
        c->Clear();
        c->cd();
    }

    TPad *pad1 = new TPad("Pad1","Pad1",0.01,0.01,0.99,0.99);
    pad1->SetFillColor(1);
    pad1->Draw();
    pad1->cd();
    gOpticsPhotonsPainter->Draw();    

}

//______________________________________________________________________________
void focalplane() {
    
    if (!gFocalSurfacePainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kFocalPlanePadID);
        gPad->Clear();
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane");
        if (!c) c = new TCanvas("c_focalplane","c_focalplane");

        c->Clear();
        c->cd();
    }
    gFocalSurfacePainter->SetCurrentMapID(kFeeHits);
    gFocalSurfacePainter->Draw("C");
    c->Modified();
    c->Update();
}

//______________________________________________________________________________
void focalplanehistos1() {

    if (!gFocalSurfaceHistoPainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kFocalPlaneHistos1PadID);
    } else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo1");
        if (!c) c = new TCanvas("c_focalplane_histo1","c_focalplane_histo1");
        c->Clear();
        c->cd();
    }

    gFocalSurfaceHistoPainter->DoGtuPhotonsHisto(c);
}

//______________________________________________________________________________
void focalplanehistos2() {

    if (!gFocalSurfaceHistoPainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kFocalPlaneHistos1PadID);
    } else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_focalplane_histo2");
        if (!c) c = new TCanvas("c_focalplane_histo2","c_focalplane_histo2");   
        c->Clear();
        c->cd();
    }

    gFocalSurfaceHistoPainter->DoGtuPhotonsHisto2(c);
}

//______________________________________________________________________________
void trigger() {
    
    if (!gTriggerPainter) return;

    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kTriggerPadID);
        gPad->Clear();
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_trigger");
        if (!c) c = new TCanvas("c_trigger","c_trigger");

        c->Clear();
        c->cd();
    }
    gTriggerPainter->SetTimeIntegral();
    gTriggerPainter->SetMapToPaintID((ETrigId)kFeeHits);
    gTriggerPainter->Draw("C");
    c->Modified();
    c->Update();
}

//______________________________________________________________________________
void truth() {
    TCanvas *c = 0;
    if (( c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_all") )) {
        c->cd(kTruthPadID);
        gPad->Clear();
    } 
    else {
        c = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("c_truth");
        if (!c) c = new TCanvas("c_truth","c_truth");

        c->Clear();
        c->cd();
    }
    TPaveText* hdemo = new TPaveText(.05,.05,.95,.95);
    hdemo->SetTextAlign(12);
    hdemo->SetTextFont(52);
    hdemo->SetTextColor(4);
    hdemo->AddText("");
    hdemo->AddText("This event was simulated with:");
    hdemo->AddText("");
    hdemo->AddText(Form("Energy %.4e MeV",gTruth->GetTrueEnergy()/1.));
    hdemo->AddText(Form("#theta      %.2f    deg",gTruth->GetTrueTheta()*TMath::RadToDeg()));
    hdemo->AddText(Form("#varphi     %.2f    deg",gTruth->GetTruePhi()*TMath::RadToDeg()));
    hdemo->AddText("");
    hdemo->AddText(Form("X_{1}       %.2f    g/cm^{2}",gTruth->GetTrueX1()*cm2/g));
    hdemo->AddText(Form("x_{init}  %.2f    km",gTruth->GetTrueInitPos().X()/km));
    hdemo->AddText(Form("y_{init}  %.2f    km",gTruth->GetTrueInitPos().Y()/km));
    hdemo->AddText(Form("x_{init}  %.2f    km",gTruth->GetTrueInitPos().Z()/km));
    hdemo->AddText("");
    hdemo->AddText(Form("X_{max}       %.2f    g/cm^{2}",gTruth->GetTrueShowerXMax()*cm2/g));
    hdemo->AddText(Form("x_{max}  %.2f    km",gTruth->GetTrueShowerMaxPos().X()/km));
    hdemo->AddText(Form("y_{max}  %.2f    km",gTruth->GetTrueShowerMaxPos().Y()/km));
    hdemo->AddText(Form("z_{max}  %.2f    km",gTruth->GetTrueShowerMaxPos().Z()/km));
    hdemo->AddText("");
    hdemo->AddText(Form("EarthAge       %.2f",gTruth->GetTrueEarthAge()));
    hdemo->AddText(Form("x_{earth}  %.2f    km",gTruth->GetTrueEarthImpact().X()/km));
    hdemo->AddText(Form("y_{earth}  %.2f    km",gTruth->GetTrueEarthImpact().Y()/km));
    hdemo->AddText(Form("z_{earth}  %.2f    km",gTruth->GetTrueEarthImpact().Z()/km));
    hdemo->AddText("");
    hdemo->Draw();

}

//______________________________________________________________________________
void animation() {

    animate = !animate;
    if(!animate) return;
    TObjArray* pads = new TObjArray;

	TPad *c1, *c2, *c3, *c4, *c_all;
	if ( (c_all = (TPad*)gROOT->GetListOfCanvases()->FindObject("c_all")) ) {
		c1 = (TPad*)c_all->FindObject(Form("c_all_%d",kShowerPadID));
		c2 = (TPad*)c_all->FindObject(Form("c_all_%d",kShowerHistoPadID));
		c3 = (TPad*)c_all->FindObject(Form("c_all_%d",kFocalPlanePadID));
		c4 = (TPad*)c_all->FindObject(Form("c_all_%d",kTriggerPadID));
	 } else {
		c1 = (TPad*)gROOT->GetListOfCanvases()->FindObject("c_shower");
		c2 = (TPad*)gROOT->GetListOfCanvases()->FindObject("c_shower_histo");
		c3 = (TPad*)gROOT->GetListOfCanvases()->FindObject("c_focalplane");
		c4 = (TPad*)gROOT->GetListOfCanvases()->FindObject("c_trigger");
	}

	
	if ( c1 ) {
		pads->Add(c1);
		gShowerPainter->Rewind();
	}
    if ( c2 ) {
        TPad* pad;
        if (( pad = (TPad*)c2->FindObject("Pad1") ))
            pads->Add(pad);
        if (( pad = (TPad*)c2->FindObject("Pad2") ))
            pads->Add(pad);
        if (( pad = (TPad*)c2->FindObject("Pad3") ))
            pads->Add(pad);
        if (( pad = (TPad*)c2->FindObject("Pad4") ))
            pads->Add(pad);
	gShowerHistoPainter->Rewind();
    }

    if ( c3 ) {
	pads->Add( c3 );
	gFocalSurfacePainter->Rewind();
    }
    
    if ( c4 ) {
	pads->Add( c4 );
	gTriggerPainter->Rewind();
    }
       
    if ( !pads->First() ) return;

    TProcessEventTimer *timer = new TProcessEventTimer(5);
    gROOT->SetInterrupt(kFALSE);
    Int_t iFrame(0);

    Printf("Animation Started");
    while( animate ) {
        if (timer->ProcessEvents()) break;
        if (gROOT->IsInterrupted()) break;

        if ( c1 && !gShowerPainter->IsEnded() )
            gShowerPainter->NextFrame();
        if ( c2 && !gShowerHistoPainter->IsEnded() )
            gShowerHistoPainter->NextFrame();
        if ( c3 && !gFocalSurfacePainter->IsEnded() && iFrame > 30 )
            gFocalSurfacePainter->NextFrame();
        if ( c4 && !gTriggerPainter->IsEnded() && iFrame > 30 )
            gTriggerPainter->NextFrame();

        TIter next(pads);
        TPad* obj = 0;
        while(( obj = (TPad*) next())) {
            obj->Modified();
            obj->Update();
        }

        
        if ( ((c1 && gShowerPainter->IsEnded())       || !c1 ) 
          && ((c2 && gShowerHistoPainter->IsEnded())  || !c2 )
          && ((c3 && gFocalSurfacePainter->IsEnded()) || !c3 )
          && ((c4 && gFocalSurfacePainter->IsEnded()) || !c4 ) ) break;
        
        iFrame++;
    }

    Printf("Animation Ended");
    delete timer;

    pads->Clear();
    delete pads;

    animate = kFALSE;
}

