// ESAF : Euso Simulation and Analysis Framework
// $Id: TestClouds.cc,v 1.23 2006/03/17 16:16:52 moreggia Exp $
// Sylvain Moreggia created Apr,  8 2004

#include "TestClouds.hh"
#include "EarthVector.hh"
#include "Config.hh"
#include "EConst.hh"
#include "EsafRandom.hh"
#include "NumbersFileParser.hh"
#include "ConfigFileParser.hh"

using EConst::EarthRadius;
using namespace sou;
using namespace TMath;

// ctor
//______________________________________________________________________________
TestClouds::TestClouds() : Clouds() {
    //
    // ctor
    //
    Configure();
    Build();
}

//______________________________________________________________________________
TestClouds::~TestClouds() {
    //
    // dtor
    //
}

//______________________________________________________________________________
void TestClouds::Configure() {
    //
    // read config files
    //
    
    // clouds features
    fType = Conf()->GetStr("TestClouds.fType");
    ConfigFileParser* pConfig3 = Config::Get()->GetCF("Atmosphere","AtmosphereFactory");
    fFeat = pConfig3->GetStr("Clouds.fFeat");    
}

//______________________________________________________________________________
void TestClouds::BuildPhaseFunction() {
    //
    // fill fTabPhFunc using tabulated phase functions (defined in config files)
    // achieve a constant step of 1deg using linear interpolation
    //

    fPhaseFunctionType = Conf()->GetStr("TestClouds.fPhaseFunctionType");  //TOFIX : should depend of clouds altitude : will be adressed soon (27/02/2006)
    if(fPhaseFunctionType == "HG" ||fPhaseFunctionType == "DHG") {
        fG1 = Conf()->GetNum("TestClouds.fG1");
        fG2 = Conf()->GetNum("TestClouds.fG2");  // only for DHG
        fF = Conf()->GetNum("TestClouds.fF");    // only for DHG
    }
    
    else if(fPhaseFunctionType == "C1cloud450") {
        NumbersFileParser nf("config/Atmosphere/PhaseFunctionData/C1cloud450",2);
        vector<Double_t> theta = nf.GetCol(0);
        vector<Double_t> value = nf.GetCol(1);
	Int_t j = 0;
        for(Int_t i=0; i < 181; i++) {
	    if(Int_t(theta[j]) == i) fTabPhFunc[i] = value[j];
	    else {
	        fTabPhFunc[i] = (value[j] - value[j-1]) / (theta[j] - theta[j-1]) * (Double_t(i) - theta[j-1]) + value[j-1];
		j--;
	    }
	    j++;
	}
    }
    
    else if(fPhaseFunctionType == "C1cloud1190") {
        NumbersFileParser nf("config/Atmosphere/PhaseFunctionData/C1cloud1190",2);
        vector<Double_t> theta = nf.GetCol(0);
        vector<Double_t> value = nf.GetCol(1);
	Int_t j = 0;
        for(Int_t i=0; i < 181; i++) {
	    if(Int_t(theta[j]) == i) fTabPhFunc[i] = value[j];
	    else {
	        fTabPhFunc[i] = (value[j] - value[j-1]) / (theta[j] - theta[j-1]) * (Double_t(i) - theta[j-1]) + value[j-1];
		j--;
	    }
	    j++;
	}
    }
    
    // needs to be normalized by 4pi
    else if(fPhaseFunctionType == "IceCrystal500") {
        NumbersFileParser nf("config/Atmosphere/PhaseFunctionData/IceCrystal500",2);
        vector<Double_t> theta = nf.GetCol(0);
        vector<Double_t> value = nf.GetCol(1);
	Int_t j = 0;
        for(Int_t i=0; i < 181; i++) {
	    if(Int_t(theta[j]) == i) fTabPhFunc[i] = value[j] / (4*Pi());
	    else {
	        fTabPhFunc[i] = (value[j] - value[j-1]) / (theta[j] - theta[j-1]) * (Double_t(i) - theta[j-1]) + value[j-1];
		fTabPhFunc[i] /= 4*Pi();
		j--;
	    }
	    j++;
	}
    }
    
    else Msg(EsafMsg::Panic) << "<BuildTabPhFunc> Invalid argument for phase function : "<< fPhaseFunctionType << MsgDispatch;
    
}

//______________________________________________________________________________
void TestClouds::Build() {
    //
    // build the object according to config files
    //
    
    TRandom* rndm = EsafRandom::Get();
    Double_t ran = 0;
    
    BuildPhaseFunction();
    
    // set by hand
    if(fFeat == "perso") {
        fOD          = Conf()->GetNum("TestClouds.fOD");
        fTopAltitude = Conf()->GetNum("TestClouds.fTopAltitude")*km;
        fThickness   = Conf()->GetNum("TestClouds.fThickness")*km;
    }


    // set according redbook
    Bool_t CS = false;
    if(fFeat == "redbook" || fFeat == "redbook_noCS") {
        // if clearsky allowed, CS set to true
	if(fFeat == "redbook") CS = true;
	do {
	    ran = rndm->Rndm();
	    if(ran <= 0.041) {
	        CS = true;
        	fOD = ran/0.041 * 0.1;
		ran = rndm->Rndm() * 0.041;
		if(ran <= 0.013)       fTopAltitude = ran/0.013 * 3*km;
		else if(ran <= 0.02)   fTopAltitude = (ran - 0.013)/(0.02 - 0.013) * (7 - 3)*km + 3*km;
		else if(ran <= 0.025)  fTopAltitude = (ran - 0.02)/(0.025 - 0.02) * (10 - 7)*km + 7*km;
		else                   fTopAltitude = (ran - 0.025)/(0.041 - 0.025) * (12 - 10)*km + 10*km;   // 12 (Liou p.180)
	    }
	    else if (ran <= 0.334) {
	        CS = true;
		fOD = (ran - 0.041)/(0.334 - 0.041) * (1 - 0.1) + 0.1;
		ran = rndm->Rndm() * 0.293;
		if(ran <= 0.056)       fTopAltitude = ran/0.056 * 3*km;
		else if(ran <= 0.078)  fTopAltitude = (ran - 0.056)/(0.078 - 0.056) * (7 - 3)*km + 3*km;
		else if(ran <= 0.114)  fTopAltitude = (ran - 0.078)/(0.114 - 0.078) * (10 - 7)*km + 7*km;
		else                   fTopAltitude = (ran - 0.114)/(0.293 - 0.114) * (12 - 10)*km + 10*km;   // 12 (Liou p.180)
	    }
	    else if(ran <= 0.479) {
	        CS = true;
		fOD = (ran - 0.334)/(0.479 - 0.334) * (2 - 1) + 1;
		ran = rndm->Rndm() * 0.145;
		if(ran <= 0.051)       fTopAltitude = ran/0.051 * 3*km;
		else if(ran <= 0.08)   fTopAltitude = (ran - 0.051)/(0.08 - 0.051) * (7 - 3)*km + 3*km;
		else if(ran <= 0.115)  fTopAltitude = (ran - 0.08)/(0.115 - 0.08) * (10 - 7)*km + 7*km;
		else                   fTopAltitude = (ran - 0.115)/(0.145 - 0.115) * (12 - 10)*km + 10*km;   // 12 (Liou p.180)

	    }
	    else if(ran <= 0.822) {
	        CS = true;
		fOD = (ran - 0.479)/(0.822 - 0.479) * (3 - 2) + 2;   // 3 is arbitrary <=> 95% absorption
		ran = rndm->Rndm() * 0.343;
		if(ran <= 0.161)       fTopAltitude = ran/0.161 * 3*km;
		else if(ran <= 0.219)  fTopAltitude = (ran - 0.161)/(0.219 - 0.161) * (7 - 3)*km + 3*km;
		else if(ran <= 0.282)  fTopAltitude = (ran - 0.219)/(0.282 - 0.219) * (10 - 7)*km + 7*km;
		else                   fTopAltitude = (ran - 0.282)/(0.343 - 0.282) * (12 - 10)*km + 10*km;   // 12 (Liou p.180)
	    }
	    else {
		fOD = 0; // means no clouds
		fTopAltitude = 0;
	    }

	    // thickness according to Liou (rough estimation)
	    ran = rndm->Rndm();
	    if(fTopAltitude == 0) fThickness = 0;
	    else if(fTopAltitude <= 3*km)  fThickness = ran * 2*km;
	    else if(fTopAltitude <= 7*km)  fThickness = ran * 4*km;
	    else                           fThickness = 2*km;
	} while(!CS);

    }
    
    // checks
    ConfigFileParser* pConfig1 = Config::Get()->GetCF("RadiativeTransfer","TestGround");
    ConfigFileParser* pConfig2 = Config::Get()->GetCF("RadiativeTransfer","RadiativeFactory");
    if(pConfig2->GetStr("RadiativeFactory.Ground") != "test") 
        Msg(EsafMsg::Warning) << "if TESTGROUND not used, be carefull of ground altitude w.r.t clouds" << MsgDispatch;
    Double_t altg = pConfig1->GetNum("TestGround.fAltitude")*km;
    if((fTopAltitude - fThickness) <= altg + kAltitudeTolerance) fThickness = fTopAltitude - altg - 10*m; // 10*m to avoid pb at interface
    if(fThickness < 0) {
        // means no clouds
        Msg(EsafMsg::Info) << "Clouds and Ground are too close -> No clouds" << MsgDispatch;
	fOD = 0;
	fTopAltitude = altg - kAltitudeTolerance;
	fThickness = 0;
    }
    
    // red book formula for clouds albedo as a function of OD
    fAlbedo = 0.11*fOD / (1 + 0.11*fOD);
    
    Msg(EsafMsg::Info) << "*** TestClouds built ***   OD = " << fOD <<" --  TopAlt = " << fTopAltitude <<" --  Thick = "<< fThickness << MsgDispatch;
}

//______________________________________________________________________________
Bool_t TestClouds::IsInClouds(const EarthVector& pos) const {
    //
    // true if position is within clouds, false otherwise
    //
    Bool_t rtn = false;
    if(fOD < kTolerance) return rtn;
    if(pos.IsUnderSeaLevel()) return false;
    if( (pos.Zv() <= (fTopAltitude + kAltitudeTolerance)) && (pos.Zv() >= (fTopAltitude - fThickness - kAltitudeTolerance)) ) rtn = true;
    
    return rtn;
}

//______________________________________________________________________________
Double_t TestClouds::TotalOD(const EarthVector& pos,const EarthVector& dir) const {
    //
    // Returns the total OD along a track defined by pos and dir
    // Case with pos within clouds handled
    // NB : vertical OD is homogeneously distributed
    //
    if(fOD < kTolerance) return 0;
    EarthVector exit, enter, direc;
    Double_t rtn(0), mag(0);
    direc = dir.Unit();
    enter = GetCloudImpact(pos,direc);
    
    // if no clouds along the track
    if(enter.Z() == HUGE) return 0;
    
    exit = GetCloudImpact(pos,direc,"outgoing");
    if(exit.Z() == HUGE) Msg(EsafMsg::Warning) << "if enter exists, exit should as well" << MsgDispatch;
    mag = (exit - enter).Mag();
    rtn = fOD * mag/fThickness;
     
    return rtn;
}

//______________________________________________________________________________
EarthVector TestClouds::RandomScatPos(const EarthVector& pos,const EarthVector& dir) const {
    //
    // returns a scattering position along a given track
    // cloud is uniform so scattering length is independent of direction : beta is constant
    // Trans = exp(-OD) = exp(-beta*l)
    //
    if(fOD < kTolerance) return EarthVector(0,0,HUGE);
    EarthVector rtn;
    EarthVector direc = dir.Unit();

    Double_t beta = fOD/fThickness;
    rtn = pos - (log(1 - EsafRandom::Get()->Rndm()) / beta)*direc;
    
    return rtn;
}

//______________________________________________________________________________
Double_t TestClouds::PhaseFunction(Double_t theta,string opt) const {
    //
    // Mie scattering phase function
    // Normalized when integrated over full solid angle
    //
    // NB : truncated phase function used only for tabulated ones. Forward peak is removed (value for theta_cut has been determined via a macro)
    //
    
    if(fOD < kTolerance) return 0.;
    Double_t rtn(0.);
    
    if(!(opt == "truncated" || opt == "real")) Msg(EsafMsg::Panic) << "<PhaseFunction> Wrong argument for option : "<< opt<< MsgDispatch;
    
    // Henyey-Greenstein parametrization
    if(fPhaseFunctionType == "HG")
        rtn = 1./(4*Pi()) * (1 - fG1*fG1) / pow(1 - 2*fG1*cos(theta) + fG1*fG1,1.5);
    
    // "Double" Henyey-Greenstein parametrization (Abu-Zayyad Astropart. 21 (2004) 163-182)
    else if(fPhaseFunctionType == "DHG") {
	Double_t HG1 = (1 - fG1*fG1) / pow(1 - 2*fG1*cos(theta) + fG1*fG1,1.5);
	Double_t HG2 = (1 - fG2*fG2) / pow(1 - 2*fG2*cos(theta) + fG2*fG2,1.5);
	rtn = 1./(4*Pi()) * (fF*HG1 + (1 - fF)*HG2);
    }
    
    // tabulated phase functions
    else if(fPhaseFunctionType == "C1cloud450") {
        Int_t theta_deg = Int_t(floor(theta*RadToDeg()));
	Double_t NORM = 1.04729;
	if(theta_deg > 180) {
	    Msg(EsafMsg::Warning) << "<PhaseFunction, tabulated> Required theta > 180deg = "<< floor(theta*RadToDeg())<< MsgDispatch;
	    theta = ACos(Cos(theta));
	    theta_deg = Int_t(floor(theta*RadToDeg()));
	}
	else if(theta_deg == 180) return fTabPhFunc[180] / NORM;
	else if(theta_deg < 3 && opt == "truncated") return 0.;
	
	// below line OK if step = 1deg
	rtn = (fTabPhFunc[theta_deg + 1] - fTabPhFunc[theta_deg]) * (theta*RadToDeg() - Double_t(theta_deg)) + fTabPhFunc[theta_deg];
	
	// may come from linear interpolation. Needed to get a normalized phase function
	rtn /= NORM;
    }
    
    // tabulated phase functions
    else if(fPhaseFunctionType == "C1cloud1190") {
        Int_t theta_deg = Int_t(floor(theta*RadToDeg()));
	Double_t NORM = 1.04015;
	if(theta_deg > 180) {
	    Msg(EsafMsg::Warning) << "<PhaseFunction, tabulated> Required theta > 180deg = "<< floor(theta*RadToDeg())<< MsgDispatch;
	    theta = ACos(Cos(theta));
	    theta_deg = Int_t(floor(theta*RadToDeg()));
	}
	else if(theta_deg == 180) return fTabPhFunc[180] / NORM;
	else if(theta_deg < 5 && opt == "truncated") return 0.;
	
	
	// below line OK if step = 1deg
	rtn = (fTabPhFunc[theta_deg + 1] - fTabPhFunc[theta_deg]) * (theta*RadToDeg() - Double_t(theta_deg)) + fTabPhFunc[theta_deg];
	
	// may come from linear interpolation. Needed to get a normalized phase function
	rtn /= NORM;
    }
    
    // tabulated phase functions
    else if(fPhaseFunctionType == "IceCrystal500") {
        Int_t theta_deg = Int_t(floor(theta*RadToDeg()));
	Double_t NORM = 1.06519;
	if(theta_deg > 180) {
	    Msg(EsafMsg::Warning) << "<PhaseFunction, tabulated> Required theta > 180deg = "<< floor(theta*RadToDeg())<< MsgDispatch;
	    theta = ACos(Cos(theta));
	    theta_deg = Int_t(floor(theta*RadToDeg()));
	}
	else if(theta_deg == 180) return fTabPhFunc[180] / NORM;
	else if(theta_deg < 2 && opt == "truncated") return 0.;
	
	// below line OK if step = 1deg
	rtn = (fTabPhFunc[theta_deg + 1] - fTabPhFunc[theta_deg]) * (theta*RadToDeg() - Double_t(theta_deg)) + fTabPhFunc[theta_deg];
	
	// may come from linear interpolation. Needed to get a normalized phase function
	rtn /= NORM;
    }
    
    else Msg(EsafMsg::Panic) << "<PhaseFunction> wrong argument for phase function type : " <<fPhaseFunctionType << MsgDispatch;

    return rtn;
}

//______________________________________________________________________________
Double_t TestClouds::PhaseFunction(const EarthVector& incom_dir,const EarthVector& outgo_dir) const {
    //
    // Mie scattering phase function
    // Normalized when integrated over full solid angle
    //
    
    if(fOD < kTolerance) return 0.;
    Double_t theta = outgo_dir.Angle(incom_dir);
    
    return PhaseFunction(theta);
}

//______________________________________________________________________________
Double_t TestClouds::TruncatedPhaseFunction(const EarthVector& incom_dir,const EarthVector& outgo_dir) const {
    //
    // Mie scattering phase function
    // Normalized when integrated over full solid angle
    //
    
    if(fOD < kTolerance) return 0.;
    Double_t theta = outgo_dir.Angle(incom_dir);
    
    return PhaseFunction(theta,"truncated");
}

//______________________________________________________________________________
Double_t TestClouds::GetMaxPhaseFunction(string opt) const {
    //
    // returns maximum value of currently used scattering phase function
    // (phase function is normalized when integrated over full solid angle)
    //
    // NB : truncated mode means the forward peak is removed (value for theta_cut has been determined via a macro)
    //
    
    if(fOD < kTolerance) return 0.;
    Double_t rtn(0.);
    
    if(opt == "real") return PhaseFunction(0.);
    else if(opt != "truncated") Msg(EsafMsg::Panic) << "<GetMaxPhaseFunction> Wrong argument for option : "<< opt<< MsgDispatch;
    
    if(fPhaseFunctionType == "HG" || fPhaseFunctionType == "DHG") return PhaseFunction(0.);
    else if(fPhaseFunctionType == "C1cloud450")         rtn = PhaseFunction(3.*DegToRad());
    else if(fPhaseFunctionType == "C1cloud1190")   rtn = PhaseFunction(5.*DegToRad());
    else if(fPhaseFunctionType == "IceCrystal500") rtn = PhaseFunction(2.*DegToRad());
    else Msg(EsafMsg::Panic) << "<GetMaxPhaseFunction> wrong argument for phase function type : " <<fPhaseFunctionType << MsgDispatch;
    
    return rtn;
}

//______________________________________________________________________________
void TestClouds::RandomDir(EarthVector& dir) const {
    //
    // 'dir' is the incoming direction -> this method set dir to the scattering outgoing direction
    // get a random direction, sampling clouds scattering phase function
    //
    
    if(fOD < kTolerance) return;
    
    Double_t theta(0);
    Double_t u1(0.), u2(0.);
    TRandom* rndm = EsafRandom::Get();
    
    // get random theta, assuming isotropic distribution in phi
    // theta is the angle between incoming and outgoing directions
    // von neumann sampling -> much faster than using TF1
    if(fPhaseFunctionType == "HG") {
	do {
            u1 = -1 + rndm->Rndm()*2;
            u2 = rndm->Rndm() * 0.5 * (1 + fG1) / (1 - fG1) / (1 - fG1);
	} while(u2 >= 0.5 * (1 - fG1*fG1) / pow(1 - 2*fG1*u1 + fG1*fG1,1.5));
    }
    
    else if(fPhaseFunctionType == "DHG") {
	Double_t HG1 = 0.;
	Double_t HG2 = 0.;
	Double_t pfvalue = 0.;
	do {
            u1 = -1 + rndm->Rndm()*2;
	    HG1 = (1 + fG1) / (1 - fG1) / (1 - fG1);
	    HG2 = (1 + fG2) / (1 - fG2) / (1 - fG2);
            u2 = rndm->Rndm() * 0.5 * (fF*HG1 + (1 - fF)*HG2);
	    HG1 = (1 - fG1*fG1) / pow(1 - 2*fG1*u1 + fG1*fG1,1.5);
	    HG2 = (1 - fG2*fG2) / pow(1 - 2*fG2*u1 + fG2*fG2,1.5);
	    pfvalue = 0.5 * (fF*HG1 + (1 - fF)*HG2);
	} while(u2 >= pfvalue);
    }
    
    else if(fPhaseFunctionType == "C1cloud450" || fPhaseFunctionType == "C1cloud1190" || fPhaseFunctionType == "IceCrystal500") {
	do {
            u1 = -1 + rndm->Rndm()*2;
	    u2 = rndm->Rndm() * PhaseFunction(0);
	} while(u2 >= PhaseFunction(ACos(u1)));
    }
    
    else Msg(EsafMsg::Panic) << "<RandomDir> wrong argument for phase function type : " <<fPhaseFunctionType << MsgDispatch;
    
    theta = acos(u1);
    
    // then get a random direction
    EarthVector v = dir.Orthogonal();
    v.Rotate(rndm->Rndm()*2*Pi(),dir);
    EarthVector axis = v.Cross(dir);
    dir.Rotate(theta,axis);
}

//_____________________________________________________________________________________________________
EarthVector TestClouds::GetCloudImpact(const EarthVector& pos, const EarthVector& dir, string opt) const {
    //
    // Impact on clouds top surface form a given (position,direction) pair in atmosphere
    // if no impact, (0,0,HUGE) is returned
    // upward bunch is handled, but no scattering to the detector, only absorption
    //
    // if opt == "default"         -> incoming impact returned
    //                                if pos is within clouds, pos is returned
    // if opt == "outgoing"        -> outgoing impact returned
    //

    // init
    EarthVector rtn(0,0,HUGE);
    if(fOD < kTolerance) return rtn;
    Bool_t isinclouds, downward;
    Double_t Altimpact(0);
    EarthVector direc = dir.Unit();
        
    // says if dir is going up/downward
    downward = GoingDownward(pos,direc);
    
    // says if pos is within the layer (altitude round-off effects < 1mm are handled)
    isinclouds = IsInClouds(pos);
    
    // set the impact surface altitude according to bunch present situation (up/downward, above/below, enter/exit clouds)
    if(isinclouds) {
        if(opt == "default") return pos;
	else if(opt == "outgoing") {
	    if(!downward) Altimpact = fTopAltitude;
	    else Altimpact = (fTopAltitude - fThickness);
	}
	else Msg(EsafMsg::Panic) << "<GetCloudImpact()> wrong argument (internal pos)" << MsgDispatch;
    }
    else {
        // if not going toward clouds
	if(pos.Zv() < (fTopAltitude - fThickness) && downward) return rtn; //TOFIX : because of earth-sphericity, impact is possible in MScatt
	if(pos.Zv() > fTopAltitude && !downward) return rtn;
	
	if(opt == "outgoing" && !downward || opt == "default" && downward) Altimpact = fTopAltitude;
	else if(opt == "outgoing" && downward || opt == "default" && !downward) Altimpact = (fTopAltitude - fThickness);
	else Msg(EsafMsg::Panic) << "<GetCloudImpact()> wrong argument (external pos)" << MsgDispatch;
    }

    // now calculate impact
    Double_t mag(0);

/*  flat earth
if(fabs(direc.Z()) < kTolerance) rtn.SetXYZ(0,0,HUGE);
else {
    mag = (Altimpact - pos.Z()) / direc.Z();
    if(mag < -kAltitudeTolerance) rtn.SetXYZ(0,0,HUGE);
    else rtn = pos + mag*direc;
}
if(mag < -kAltitudeTolerance) Msg(EsafMsg::Warning) << "PB in impact calculation GetCloudImpact(), mag = ["<<mag<< MsgDispatch;
*/

    Double_t b = pos*direc + direc.Z()*EarthRadius();
    Double_t c = pos.Mag2() + 2*EarthRadius()*(pos.Z() - Altimpact) - pow(Altimpact,2);
    pair<Int_t,Double_t*>& p = findRoots(1.,2*b,c);
    if(p.first == 0) {
#ifdef DEBUG
    //Msg(EsafMsg::Debug) <<"NO IMPACT ON Clouds" << MsgDispatch;
#endif
	return rtn;
    }
    if(p.first == 1) mag = p.second[0];
    else if(p.first ==2) {
	if((p.second[0]+kAltitudeTolerance) * (p.second[1]+kAltitudeTolerance) < 0) mag = max(p.second[0],p.second[1]);
	else mag = min(p.second[0],p.second[1]);
    }
    if(mag < -kAltitudeTolerance) Msg(EsafMsg::Warning) << "PB in impact calculation GetCloudImpact(), [min,max] = ["<<mag<<", "<< max(p.second[0],p.second[1])<<"]" << MsgDispatch;
    rtn = pos + mag*direc;    


    return rtn;
}

//_____________________________________________________________________________
Bool_t TestClouds::GoingDownward(const EarthVector& pos,const EarthVector& direc) const {
    //
    // to know if locally track is going up/downward
    // locally means at pos nadir
    // if perfectly horizontal (i.e. local angle of halfpi) returns false (i.e. upward)
    //
    if(fOD < kTolerance) return false;
    Bool_t rtn;

/*   flat earth
if(direc.Z() > 0) return false;
else return true;
*/

    EarthVector dir = direc.Unit();
    // find local zenith angle
    Double_t angle;
    EarthVector temp(pos);
    temp(2) += EarthRadius();  // pos expressed in earth-centered frame -> gives local vertical direction (expressed in MES frame)
    angle = dir.Angle(temp);    // angle between dir and local vertical
    // returns accordingly
    if(angle < (TMath::PiOver2() - kTolerance)) rtn = false;
    else  rtn = true;

    return rtn;
}

//_____________________________________________________________________________
void TestClouds::Reset() {
    //
    // reset clodus features
    //
    
    fOD = 0;
    fThickness = 0;
    fTopAltitude = 0;
    fAlbedo = 0;
    
    Build();
}


ClassImp(TestClouds)
