// ESAF : Euso Simulation and Analysis Framework
// $Id: HmaxByShapeMethod.cc,v 1.3 2006/03/15 10:50:36 naumov Exp $
// Dmitry V. Naumov created Jun,  6 2004

#include "HmaxByShapeMethod.hh"
#include "Atmosphere.hh"
#include "utils.hh"
#include "RecoEvent.hh"
#include "EDetector.hh"
#include "EConst.hh"
#include "EnergyTypes.hh"
#include "EnergyViewer.hh"

#include <TH1F.h>
#include <TF1.h>
#include <TMinuit.h>

#include <iostream>

using namespace sou;
using namespace TMath;
using namespace EConst;
using namespace std;

ClassImp(HmaxByShapeMethod)
// fFluorescence function
//_____________________________________________________________________________
Double_t HSM_fFluorescence(Double_t *x, Double_t *par) {
    return par[0]*exp(-0.5*pow(x[0]-par[1],2)/pow(par[2],2));
}

//_____________________________________________________________________________
Double_t HSM_Background(Double_t *x, Double_t *par) {
    //
    //
    //

    Float_t time    = x[0];
    HmaxByShapeMethod* hm = (HmaxByShapeMethod*)gMinuit->GetObjectFit();
    EnergyModule *fEnergyModule = hm->GetEnergyModule();
    if (!fEnergyModule) {
      TF1::RejectPoint();
      return 0;
    }

    Float_t Theta = fEnergyModule->ThetaFoV(time);

    if (Theta*RadToDeg() > 30.) {
      TF1::RejectPoint();
      return 0;
    }

    GtuPixels_t GtuPixels = fEnergyModule->GetMyGtuPixels();
    Float_t gtulength = fEnergyModule->GetRecoEvent()->GetHeader().GetDetector()->GetElectronics()->GetGtuLength()/microsecond;
    Int_t gtubin = (Int_t)(time/gtulength);
    Pixels_t OneGtuPixels = GtuPixels[gtubin];
    Float_t background = fEnergyModule->BgrPerPixelPerGtu(time)*OneGtuPixels.size();

    return background;
}

// Sum of background and peak functions
//_____________________________________________________________________________
Double_t HSM_fitFunction(Double_t *x, Double_t *par) {
    return HSM_fFluorescence(x,&par[0]) ;//+ HSM_Background(x,&par[0]);
}

//_____________________________________________________________________________
float HSM_DensityToAltitude(float h) {
    HmaxByShapeMethod* hm = (HmaxByShapeMethod*)gMinuit->GetObjectFit();
    return Atmosphere::Get()->Air_Density(h) - hm->GetDensity();
}

//_____________________________________________________________________________
HmaxByShapeMethod::HmaxByShapeMethod(EnergyModule* em): EsafMsgSource() {
    //
    // ctor
    //
    if (!em) Msg(EsafMsg::Panic)<< "Pointer to EnergyModule is NULL. I do not want to work like this" <<MsgDispatch;

    fEnergyModule  = em;
    fEnergy        = 1.e14;
    fFitFcn        = NULL;
    fSigma         = -HUGE;
    fAltitude      = -HUGE;
    fDensity       = -HUGE;
}

//_____________________________________________________________________________
HmaxByShapeMethod::~HmaxByShapeMethod() {
    //
    // dtor
    //
    Clear();
}

//_____________________________________________________________________________
void HmaxByShapeMethod::Clear()  {
    //
    // clean 
    //
    SafeDelete(fFitFcn);
}

//_____________________________________________________________________________
Bool_t HmaxByShapeMethod::Fit(TH1F* data) {
    //
    // fit entry histogramm and estimate altitude of the shower maximum. The entry histo is assumed to be unfolded of inefficients  
    //
    Clear();
    Double_t par[3]={1,1,1};
    gMinuit->SetObjectFit(this);
    RecoEvent *ev = fEnergyModule->GetRecoEvent();
    Int_t fFirstHit = ev->GetStartGtu();
    Int_t fLastHit  = ev->GetEndGtu();
    fFitFcn = new TF1("fFitFcn",HSM_fitFunction,fFirstHit,fLastHit,3);
    fFitFcn->SetNpx(500);
    fFitFcn->SetLineWidth(2);
    fFitFcn->SetLineColor(kMagenta);

    fFitFcn->SetParLimits(0,0,data->GetMaximum());
    fFitFcn->SetParLimits(1,0,ev->GetEndGtu());
    fFitFcn->SetParLimits(2,0,0.5*ev->GetEndGtu()); 
    fFitFcn->SetParameter(1,fFluorMaxTime);

    data->Fit("fFitFcn","RQ");
    fChi2 = data->GetFunction("fFitFcn")->GetChisquare()/data->GetFunction("fFitFcn")->GetNDF();
    data->GetFunction("fFitFcn")->GetParameters(&par[0]);
    fFitFcn->SetParameters(par);
    fFluorMaxTime = par[1];

    // Compute altitude of the shower maximum
    Float_t x0 = 37.15*g/cm2;
    Float_t t0 = 1.7+0.76*Log(fEnergy/81.);                   // energy in MeV !!!
    Float_t gtulength = fEnergyModule->GetRecoEvent()->GetHeader().GetDetector()->GetElectronics()->GetGtuLength();
    fSigma = par[2]*gtulength;
    fDensity = Sqrt(2*t0)*x0*(1.+fShowerDir*fShowerMaxDir)/Clight()/fSigma;
    DensityToAltitude();
    Msg(EsafMsg::Debug) << "Reconstructed altitude of the shower maximum "<< fAltitude/km <<MsgDispatch;
    Msg(EsafMsg::Debug) << "Reconstructed density at the shower maximum " << fDensity/g*cm3 << " sigma " << fSigma/microsecond << MsgDispatch;

    Int_t debug = fEnergyModule->GetDebugLevel();
    if (debug>=1) {
        EnergyViewer *fViewer = fEnergyModule->GetEnergyViewer();
        fViewer->DisplayAltitude(data,fFitFcn);
    }
    return kTRUE;
}

//_____________________________________________________________________________
void HmaxByShapeMethod::DensityToAltitude() {
    //
    // Finds the altitude which corresponds to the given density
    Float_t h1 = 0*km, h2 = 100*km; 
    Float_t (*f)(float)= &HSM_DensityToAltitude;
    zbrac(f,&h1,&h2);
    fAltitude = rtbis(f,h1,h2,0.01*km);
}


//_____________________________________________________________________________
void HmaxByShapeMethod::Set(Float_t energy,Float_t timemax, TVector3 showerdir, TVector3 showermaxdir){
    //
    // Set up the module
    // 
    SetEnergy(energy);
    SetFluoMaxTime(timemax);
    SetShowerDir(showerdir);
    SetShowerMaxDir(showermaxdir);
}
