// $Id: SimpleDetector.hh,v 1.1 2006/03/24 16:23:27 moreggia Exp $
// Author: Sylvain Moreggia   2006/03/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SimpleDetector                                                           *
 *  Package: electronics                                                   *
 *  Coordinator: Marco.Pallavicini                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __SIMPLEDETECTOR_HH__
#define __SIMPLEDETECTOR_HH__

#include "euso.hh"
#include "Detector.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// SimpleDetector                                                                 //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class PhotoElectron;
class Photon;


class SimpleDetector :  public Detector,  public EsafConfigurable, public EsafMsgSource {
public:
    
    // ctor
    SimpleDetector();
    
    // dtor
    virtual ~SimpleDetector();

    // do the complete simulation of one event
    virtual Telemetry* Get( PhotonsOnPupil* );

    // get ready for next event
    virtual void Reset();

    // initialization
    void InitDetector();

    // apply optics effect
    Int_t ApplyOpticsFilter(const Photon& ph) const;
    
    // Create PhotoElectrons using Detection Quantum Efficiency (= Collection eff. * Quantum eff.)
    Int_t ApplyDQE(const Photon& ph) const;
    
    // interpolation functions according to wl
    Double_t InterpolateBG3(Double_t) const;
    Double_t InterpolateQE(Double_t) const;
    
    // orders pe- chronologically
    // then find the time of first GTU and ranges them in the map using GTU info
    // set pixel ID as well
    void SpaceTimePEinfos(const vector<PhotoElectron*>&);
    
    // find pe- pixel
    // - using sqrt(x*x + y*y) value in the MES Z=0 plane
    // - i,j in [0..n-1]  (i<-->row    j<-->column)
    // - pixeldID = N --> i,j coordinates found back using N / n = i  and  N % n = j
    UInt_t FindPixelID(const PhotoElectron&) const;
    
    // add background in pixels which contain pe- from signal photons
    // returns nb of bgnd pe-
    Int_t AddBackground();
    
    // kind of trigger simulation
    // Tag pe- according to the SNR per pixel per GTU : if (S + B) >= (<B> + 3*sigmaB), pixel/GTU considered as triggered
    // returns nb of triggered pe-
    Int_t ApplyTrigger();
    
    // calculate mean background per pixel per GTU (Nadir incidence)
    Double_t MeanBgndPerPixelPerGTU(Double_t);
    
    // getters
    UInt_t GetNbPhotoElectrons() const {return fPhotoElectrons.size();}
    
    // physically release the memory allocated by the arrays of this object
    virtual Bool_t ClearMemory() {return kTRUE;}


    EsafConfigClass(Electronics,SimpleDetector)
    
private:

    map<UInt_t, vector<PhotoElectron*>* > fPhotoElectrons;      // list of photoelectrons, stored per GTU (start at 0)
    Double_t fBG3filter[400];                                   // tabulated value for BG3 filter
    Double_t fPMQE[51];                                         // tabulated values for PM Quantum Efficiency
    Double_t fGTUlength;                                        // GTU length
    Double_t fPixelSideLength;                                  // pixel side length projected on the MES plane Z=0
    Double_t fFirstPixelX;                                      // pixel high left corner X-position projected on the MES plane Z=0
    Double_t fFirstPixelY;                                      // pixel high left corner Y-position projected on the MES plane Z=0
    UInt_t fNbPixelOnSide;                                      // nb of pixels on a side of the square FS
    Double_t fMeanBgnd;                                         // value in ph-on-pupil/m2/ns/sr
    Double_t fTriggerLevel;                                     // nb of background sigma considered for SNR treatment
    string fBgndMode;                                           // RedBook (within 300-400nm) or Extended (extrapolation within 200-600nm)
    string fBG3type;                                            // BG3 (long tail above 400nm), or BG3_coated (RedBook config, sharp cut above 400nm)
    
    UInt_t fNbGTUs;                                             // nb of GTUs containing pe-
    UInt_t fNbSignalPE;                                         // nb of pe- coming from signal photons
    Double_t fFirstGTUTime;                                     // time of first GTU (raising time)
    Double_t fFirstPETime;                                      // time of first pe-
    Double_t fLastPETime;                                       // time of last pe-

    ClassDef(SimpleDetector,0)
};

#endif  /* __SIMPLEDETECTOR_HH__ */

