// $Id: MCRadiativeTransfer.cc,v 1.42 2006/03/24 16:23:27 moreggia Exp $
// Author: Sylvain Moreggia   2005/08/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: MCRadiativeTransfer                                                  *
 *  Package: radiativetransfer                                               *
 *  Coordinator: Sylvain Moreggia                                            *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// MCRadiativeTransfer
//
// Detection from a ground detector is now possible in this mode
//
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//
//   - fMScattAnalysis >0 means Detector geometry is completely switched off
//         - no direct photons
//         - scattered SinglePhotons are located at their last scattering position
//         - it is a pure Multiple Scattering Analysis : no propagation to detector (actually there is, but transmissions=1)
//         - MaxPhi=1
//         - MaxOmega=1e-20 for fPropagator and =1 for PhotonsInOmega()
//         - if == 1 : simu checks if photon goes out freely of atmosphere. If it does, it is counted, if not it is lost
//         - if == 2 : does not check -> allow to study scattering without being biased by last transfer
//

#include "MCRadiativeTransfer.hh"
#include "Atmosphere.hh"  
#include "RadiativeFactory.hh"
#include "ListPhotonsInAtmosphere.hh"
#include "ListPhotonsOnPupil.hh"
#include "EsafRandom.hh"
#include <TROOT.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TProfile.h>
#include <math.h>  
#include "ParentPhoton.hh" 
#include "SinglePhoton.hh" 
#include "Config.hh"
#include "LowtranRadiativeProcessesCalculator.hh"
#include "FastRadiativeProcessesCalculator.hh"
#include "EConst.hh"
#include "BunchOfPhotons.hh"
#include "Ground.hh"

#include "EEvent.hh"
#include "EAtmosphere.hh"
#include "EAtmosphereBunchAdder.hh"
#include "EAtmosphereSingleAdder.hh"

#include "TBenchmark.h"

ClassImp(MCRadiativeTransfer)

using namespace TMath;
using namespace sou;
using namespace EConst;

//_____________________________________________________________________________
MCRadiativeTransfer::MCRadiativeTransfer() : RadiativeTransfer() , fPhotons(0), fTotalList(0), fPropagator(0), fLowToDetec(0),
                                             fNbBunch(0), fNbTot(0), fNbSingles(0), fOmegaMax(0), fDistance_cut(0) {
    //
    // ctor
    //
    Msg(EsafMsg::Info) << "Enabled" << MsgDispatch;
    fGround = RadiativeFactory::Get()->GetGround();
    if(!fGround) Msg(EsafMsg::Panic) << "Pb of memory allocation for fGround" << MsgDispatch;
    
    Double_t maxtof = Conf()->GetNum("MCRadiativeTransfer.fTofCut")*microsecond;
    fMScattAnalysis = Int_t(Conf()->GetNum("MCRadiativeTransfer.fMScattAnalysis"));
    fKeepAll = Conf()->GetBool("MCRadiativeTransfer.fKeepAll");
    fPropagator = new SinglePhotonPropagator(fGround,maxtof);
    fLowToDetec = new LowtranRadiativeProcessesCalculator();
    
    
    fMaxPhNb = Conf()->GetNum("MCRadiativeTransfer.fMaxPhNb");
    
    fScatOrder = Int_t(Conf()->GetNum("MCRadiativeTransfer.fScatOrder"));
    Msg(EsafMsg::Info) << "Max Scattering order to be simulated = " << fScatOrder << MsgDispatch;
    
    // detector geometry : decoupled or optimized
    // decoupled : - detector seems to be a sphere. Allows to study several detector geometries with the same RadiativeTransfer simulation
    //             - MUST be used in ground detector mode
    // optimized : - true detector geometry is used in RadiativeTransfer simulation
    //             - not available for ground detector
    string name = Conf()->GetStr("MCRadiativeTransfer.fDecoupled");
    if(name == "decoupled") fDecoupled = true;
    else if(name == "optimized") fDecoupled = false;
    else Msg(EsafMsg::Panic) << "Wrong option for MCRadiativeTransfer.fDecoupled" << MsgDispatch;
    
}

//_____________________________________________________________________________
MCRadiativeTransfer::~MCRadiativeTransfer() {
    //
    // Destructor
    //
    SafeDelete(fPhotons);
    SafeDelete(fPropagator);
    SafeDelete(fLowToDetec);
}

//______________________________________________________________________________
Bool_t MCRadiativeTransfer::ClearMemory() {
    // 
    // Release all the mem hold in the buffers
    //

    Reset();
    vector<SinglePhoton*> emptySingle;
    emptySingle.swap(fTempList);
    
    return fPhotons ? fPhotons->ClearMemory() : kTRUE;
}


//_______________________________________________________________________________________________________________________
PhotonsOnPupil* MCRadiativeTransfer::Get(PhotonsInAtmosphere* photons, const DetectorGeometry* dg) {
    //
    // Transport photons from source to Euso pupil
    //



    ////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////// Initializations /////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////


// 1. Preprocesses
    // copy DetectorGeometry
    CopyDetectorGeometry(dg);
    
    // in case ground detector simulation
    if(EUSO().Zv() == 0) fDetAtGrnd = true;
    else fDetAtGrnd = false;
    if(fDetAtGrnd && !fDecoupled) Msg(EsafMsg::Panic) << "In Ground detector mode, decoupled mode must be switched on" << MsgDispatch;
    if(fDetAtGrnd) Msg(EsafMsg::Warning) << "Ground detector mode enabled" << MsgDispatch;
    
    // if given list of photons is empty  
    if(!photons) return NULL;

    if(photons->GetType() == "empty") Msg(EsafMsg::Panic) <<"When NoLightSource used, NoRadiativeTransfer should be used"<< MsgDispatch;
    if(photons->GetType() != "list") Msg(EsafMsg::Panic) <<"Wrong PhotonsInAtmosphere format. ListPhotonsInAtmosphere expected."<< MsgDispatch;
    fTotalList = (ListPhotonsInAtmosphere*) photons;


    // Build a datacard of VERTICAL Lowtran transmission for PropagateAllToDetector() and PropagateTempListToDetector(..) method
    // NB : internally checks if datacard already exists. If so, does not rebuild it
    if(!fDecoupled && !fMScattAnalysis) {
        Msg(EsafMsg::Info) <<"Building LOWTRAN vertical transmission datacard"<< MsgDispatch;
        fLowToDetec->MakeVerticalDatacard(fGround,EUSO().Zv());
        Msg(EsafMsg::Info) <<"LOWTRAN datacard built"<< MsgDispatch;
    }
    else Msg(EsafMsg::Warning) <<"NO LOWTRAN datacard in decoupled mode (including ground mode) or MScattAnalysis mode"<< MsgDispatch;




// 2. Get some infos about created light
    // look at number of bunches and photons
    fNbBunch = fTotalList->GetListOfBunch().size();
    fNbTot = 0;
    fNbSingles = 0;
    Double_t nf(0), nc(0);
    for(size_t i=0; i<fNbBunch; i++) {
        fNbTot += (fTotalList->GetListOfBunch()[i])->GetWeight();
        if(fTotalList->GetListOfBunch()[i]->GetType() == Fluo)
	     nf += fTotalList->GetListOfBunch()[i]->GetWeight();
        else nc += fTotalList->GetListOfBunch()[i]->GetWeight();
    }

// some prints
#ifdef DEBUG
    Msg(EsafMsg::Debug) << "SIZE OF LIST OF BUNCHES = "<<fNbBunch << MsgDispatch;
    Msg(EsafMsg::Debug) << "Nb fluo = " <<nf << MsgDispatch;
    Msg(EsafMsg::Debug) << "Nb ckov = " <<nc << MsgDispatch;

    if ( fNbBunch >=1 ) {
        EarthVector track = fTotalList->GetListOfBunch()[fNbBunch - 1]->GetPos() 
                          - fTotalList->GetListOfBunch()[0]->GetPos();
        Msg(EsafMsg::Debug) << "size of the photon track = "<<track.Mag()/km <<" km"<< MsgDispatch;
    }
#endif
    Msg(EsafMsg::Info) << "TOTAL number of photons ="<<fNbTot << MsgDispatch;
    
    
    
    
    
// 3. Prepare Monte Carlo algorithm
    // set omega max
    if(fMScattAnalysis == 0) {
        // fOmegaMax setting for space detector : last scattering layer is taken at 30km altitude
	//TOFIX : if a scattering exists higher in altitude
        fOmegaMax = EusoOmega(EarthVector(0,0,30*km));
        if(fDetAtGrnd) {
	    fDistance_cut = Conf()->GetNum("MCRadiativeTransfer.fDistance_cut")*km;
	    fOmegaMax = EusoOmega(EarthVector(0,0,fDistance_cut));
	}
        fPropagator->SetOmegaMax(fOmegaMax);
    }
    else {
        fOmegaMax = 1;
        fPropagator->SetOmegaMax(1e-20);  // means that fPropagator will keep every photon that reaches the relevant scat order
    }
    
    
    // set phase function maximum value for the set of phase functions involved in the simulation :
    // molecular, ground, clouds, (aerosols to come)
    if(fMScattAnalysis == 0) {
        Double_t molecular_value = fPropagator->GetRadiativeCalculator()->GetMaxPhaseFunction("rayleigh");
        Double_t clouds_value = Atmosphere::Get()->GetClouds()->GetMaxPhaseFunction("truncated");
        Double_t ground_value = fGround->GetMaxOutgoing_phase_function();
        fMaxPhaseFunction = Max(molecular_value,clouds_value);
        fMaxPhaseFunction = Max(fMaxPhaseFunction,ground_value);
        Msg(EsafMsg::Info) << "MAXIMUM OF PHASE FUNCTION = "<<fMaxPhaseFunction << MsgDispatch;
    }
    else fMaxPhaseFunction = 1;
    fPropagator->SetMaxPhaseFunction(fMaxPhaseFunction);




// 4. root output init
    EEvent* ev = EEvent::GetCurrent();
    if(ev->GetAtmosphere()) ev->GetAtmosphere()->SetMaxScatOrder(fScatOrder);

    // if some SinglePhotons created directly by LightSource module, they are saved into root
    if ( ev ) {
        const vector<SinglePhoton*>& list_single_2 = fTotalList->GetListOfSingle();
        size_t nbnotsaved = fTotalList->GetNbNotSaved();
        for (size_t i = list_single_2.size() - nbnotsaved; i<list_single_2.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single_2[i],true,false,0 );
            ev->Fill(sa);
            fTotalList->OneSingleSaved();
        }
    }




    ////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////// ALGO STARTS HERE /////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////
    BunchOfPhotons* bunch = 0;
    Bool_t next(0),subnext(0);
    Double_t nbTracked(0), fraction(0), left(0),right(10),subleft(0),subright(2.5);
    Int_t split_level = Int_t(ceil(fNbTot * fOmegaMax * fMaxPhaseFunction / fMaxPhNb)) + 1;  // +1 in case of round-off
    Msg(EsafMsg::Warning) << "NB of photons to propagate after reduction "<<fNbTot * fOmegaMax * fMaxPhaseFunction<< MsgDispatch;
    Msg(EsafMsg::Warning) << "Each scattering order simu will be split in "<<split_level<<" parts" << MsgDispatch;
    
    // 1. Direct photons simulation
    while(true) {
        bunch = fTotalList->GetBunch();
        if(!bunch) break;

        // saves bunch parameters at creation
        if ( ev ) {
            EAtmosphereBunchAdder ba( bunch, true );
            ev->Fill(ba);
        }

        // photons going directly toward detector
        if(!fMScattAnalysis) DirectToEuso(*bunch);  // no direct light in MScattAnalysis mode
    }

    // 2.3 Scan fTempList before filling fTotalList, applying relevant cuts :
    //     - if photon status > 4, it is lost
    //     - if GROUND detector, check if photon is a real candidate. If not, it is lost
    //     - if fKeepAll == false, photons are propagated to detector and those absorbed are lost
    Msg(EsafMsg::Info)  <<"Nb of direct photons before cuts = " <<fTempList.size() << MsgDispatch;
    ScanTempList();
    fTotalList->Add(fTempList);
    


    // 2. SCATTERING SIMULATION loop
    for(Int_t ord=1; ord <= fScatOrder; ord++) {
        Msg(EsafMsg::Info) << "\nSCATTERING ORDER = " << ord<< MsgDispatch;
#ifdef DEBUG
        TString jobMCRT = "Time spent for Scattering simu:";
        TBenchmark gB;
        gB.Start(jobMCRT);
#endif
        nbTracked = 0;
        fraction = 0;
        left = 0;
        right = 10;
        subleft = 0;
        subright = 2.5;
        fTotalList->ResetBunchCounter();  // for below extraction from list
        Int_t loop_j = 0;
        Int_t remainingPhotons = -1;
	for(Int_t level=0; level < split_level; level++) {
            Msg(EsafMsg::Info) << "************ Starting bunches transformation Level nb "<<level+1<<" ************"<< MsgDispatch;
            if(level == 0) Msg(EsafMsg::Info) << " : 0%" << MsgFlush;
            for(UInt_t i=loop_j; i <fTotalList->GetBunchEntries(); i++) {
        	bunch = fTotalList->GetBunch(loop_j);
		loop_j++;
        	nbTracked++;

        	// 2.1 generate reduce nb of photons
        	remainingPhotons = PhotonsInOmega(*bunch,remainingPhotons);
		
		// if huge nb of photons (>1e6), present scattering order simu is splitted
		if(remainingPhotons > -1) {
		    Msg(EsafMsg::Warning) << "\nSplit level nb " <<level+1<<" reached" << MsgDispatch;
		    loop_j--;
        	    nbTracked--;
		    break;
		}

        	// dump info
        	fraction = 100*nbTracked/fNbBunch;
        	if (left<fraction && fraction <=right) 
                    next = kFALSE;
        	else if (fraction>right) {
                    next = kTRUE;
                    left = right;
                    right += 10;
        	}
        	if (subleft<fraction  && fraction <=subright) {
                    subnext = kFALSE;
        	}
        	if (fraction > subright) {
                    subnext = kTRUE;
                    subleft = subright;
                    subright +=2;
        	}
        	if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        	if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
            }
            Msg(EsafMsg::Info) << " -done" << MsgDispatch;
            Msg(EsafMsg::Info) << "NB of SinglePhoton to PROPAGATE = "<<fNbSingles<< MsgDispatch;
            fNbSingles = 0;  // reset


            // 2.2 scattering simu for this bunch at the relevant scattering order
            fPropagator->Go(fTempList,ord);

	    // 2.2' in MScattAnalysis MODE 1
	    // check if photons would scatter at next order
	    // if not they are KEPT, otherwise they are flagged as "LostByCut"
	    if(fMScattAnalysis == 1) fPropagator->CheckIfScatAgain(fTempList);


	    // 2.3 Scan fTempList before filling fTotalList, applying relevant cuts :
	    //     - if photon status > 4, it is lost
	    //     - if GROUND detector, check if photon is a real candidate. If not, it is lost
	    //     - if fKeepAll == false, photons are propagated to detector and those absorbed are lost
	    ScanTempList();

            // 2.4 Move photons from fTempList to fTotalList : fTempList does not contain photons anymore
            //     Only photons which have successfully scattered are written in fTotalList, others are lost
            fTotalList->Add(fTempList);
	    
#ifdef DEBUG
            gB.Stop(jobMCRT);
            MsgForm(EsafMsg::Info,"Time spent for this scattering order :  REAL=%6.2f s   CPU=%6.2f s",gB.GetRealTime(jobMCRT),gB.GetCpuTime(jobMCRT));
#endif
	}
    }




    // 3. saves single photon parameters after scattering simulation
    if ( ev ) {
        const vector<SinglePhoton*>& list_single = fTotalList->GetListOfSingle();
        size_t nbnotsaved = fTotalList->GetNbNotSaved();
        for (size_t i = list_single.size() - nbnotsaved; i<list_single.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single[i],true,true );
            ev->Fill(sa);
            fTotalList->OneSingleSaved();
        }
    }

#ifdef DEBUG
    Msg(EsafMsg::Debug) <<"final list of single, size = " <<fTotalList->GetListOfSingle().size() <<MsgDispatch;
#endif
    if(fTotalList->GetNbNotSaved()) Msg(EsafMsg::Warning) <<fTotalList->GetNbNotSaved()<<" SinglePhoton not saved into root" <<MsgDispatch;




    // 4. propagates all direct and scattered photons until detector (Lowtran is used, c.f. Ozone)
    //    ALLOW to study transmission values along the last travel to detector
    //    IS fake if fMScattAnalysis > 0
    if(fKeepAll) PropagateAllToDetector();
        
    
    // 5. Convert PinA in PonP
    ConvertIntoPonP();

    // saves single photons after propagation until detector
    if ( ev ) {
        const vector<SinglePhoton*>& list_single_3 = fTotalList->GetListOfSingle();
        for ( size_t i=0; i<list_single_3.size(); i++ ) {
            EAtmosphereSingleAdder sa( list_single_3[i],false,false,i );
            ev->Fill(sa);
        }
    }
    

    return fPhotons;
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::Reset() {
    //
    // get ready for next event
    // NB : fLowToDetec not reset, its datacard is used for all the events of a run
    //
    if(fGround) fGround->Reset();
    if(fPhotons) fPhotons->Clear();
    if(fPropagator) fPropagator->Reset();
    for(size_t i=0; i<fTempList.size(); i++) {
        SafeDelete(fTempList[i]);
        Msg(EsafMsg::Panic) <<"fTempList SHOULD BE EMPTY, remains : "<<fTempList.size()<<" photons inside"<<MsgDispatch;
	
    }
    if(fLowToDetec) fLowToDetec->Reset();
    fTempList.clear();
    fNbBunch = 0;
    fNbTot = 0;
    fNbSingles = 0;
    fOmegaMax = 0;
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::DirectToEuso(const BunchOfPhotons& b) {
    //
    // Creates a list of SinglePhoton considering the EUSO solid angle
    // handles fluo and cerenkov (angular distrib used for the later)
    // THESE SINGLEPHOTON WILL NOT UNDERGO SCATTERING SIMU
    //
    
    Int_t nb = 0;
    EarthVector towardEUSO = (EUSO() - b.GetPos()).Unit();
    Double_t theta = fabs(towardEUSO.Angle(b.GetDir())); // to keep theta within 0-Pi()
    if(theta > Pi()) Msg(EsafMsg::Warning) << "<DirectToEuso> Pb with angle definition" << MsgDispatch;
    
    Double_t angular_distrib_value = b.AngularDist_OverTwoPi(theta);
    nb = EsafRandom::Get()->Poisson(b.GetWeight() * EusoOmega(b.GetPos()) * angular_distrib_value);

    // SinglePhotons created and added to fTotalList (THEY WILL NOT UNDERGO SCATTERING SIMU)
    Double_t wl, date, tof;
    EarthVector showerpos, dir, diff;
    SinglePhoton* s = 0;
    
    UInt_t bid = b.GetId();
    PhotonType type = b.GetType();
    tof = 0;
    
    for(Int_t i=0; i<nb; i++) {
        // corrections for date and showerpos from BunchOfPhotons mean values and longitudinal dispersion
        showerpos = b.RandomPosInShower();
	if(fGround->IsUnderGround(showerpos)) continue;
	diff = showerpos - b.GetShowerPos();
        date = b.GetDate() + diff.Dot(b.GetDir().Unit())/Clight();
        dir = EUSO() - showerpos;
	wl = b.GetWlSpectrum().GetLambda();
	s = new SinglePhoton(type,date,tof,wl,showerpos,showerpos,dir,Direct,bid);
	fTempList.push_back(s);
    }
}

//_______________________________________________________________________________________________________________________
Int_t MCRadiativeTransfer::PhotonsInOmega(const BunchOfPhotons& b, Int_t remainingPhotons) {
    //
    // Detector solid angle is applied here -> the resulting photons will undergo scattering simu
    // Photon features are set here (lambda, position, direction)
    // SinglePhoton are stored into fTempList for scattering simu
    //
    // if remainingPhotons > -1, means the nb of photons to create is specified
    //

    Int_t nb = 0;
    Int_t rtn = -1;
    Double_t wl, date, tof;
    EarthVector showerpos, dir, diff;
    SinglePhoton* s = 0;
    UInt_t bid = b.GetId();
    PhotonType type = b.GetType();
    tof = 0;
    
    if(fMScattAnalysis) nb = Int_t(b.GetWeight());
    else if(remainingPhotons > -1) nb = remainingPhotons;
    else nb = EsafRandom::Get()->Poisson(b.GetWeight() * fOmegaMax * fMaxPhaseFunction);
    
    for(Int_t i=0; i<nb; i++) {
        // get features at creation
        showerpos = b.RandomPosInShower();
        if(fGround->IsUnderGround(showerpos)) continue;
        diff = showerpos - b.GetShowerPos();
        date = b.GetDate() + diff.Dot(b.GetDir().Unit())/Clight();
        dir  = b.RandomDirection();
        wl   = b.GetWlSpectrum().GetLambda();
        s    = new SinglePhoton(type,date,tof,wl,showerpos,showerpos,dir,None,bid);
        fNbSingles++;
        fTempList.push_back(s);
	if(fNbSingles >= fMaxPhNb) {
	    rtn = nb - 1 - i;
	    break;
	}
    }
    
#ifdef DEBUG
    //Msg(EsafMsg::Debug)  <<"  InOmega gives -> " <<nb <<" photons" << MsgDispatch;
#endif

    return rtn;
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::PropagateAllToDetector() {
    //
    // Final phase of transfer. Propagate all the SinglePhoton til EUSO pupil
    // USE LOWTRAN (because of Ozone transmission)
    // DISABLED if fKeepAll == false, PropagateTempListToDetector() method used instead
    // FAKE if fMScattAnalysis > 0
    //

    EarthVector dirtest(1);
    Double_t Trans[4];
    Double_t TotTrans(0.);
    TRandom* rndm = EsafRandom::Get();
    SinglePhoton* p = 0;
    size_t nTotal(0),nTracked(0);
    Float_t fraction(0); 
    Bool_t next(0),subnext(0);
    Float_t left(0),right(10),subleft(0),subright(2.5);
    nTotal = fTotalList->GetSingleEntries();
    Msg(EsafMsg::Info) << "Propagation All photons to Detector: 0%" << MsgFlush;
    FastRadiativeProcessesCalculator* fastc = 0;

    if(!fMScattAnalysis && fDetAtGrnd) fastc = new FastRadiativeProcessesCalculator();

    // loop over the list of SinglePhoton
    while(true) {
        p = fTotalList->GetSingle();
        if(!p) break;
        nTracked++;
		
	// photon direction is toward EUSO
        p->SetDir((EUSO() - p->Pos()).Unit());

        // calculate transmission from photon position to EUSO
	// for ground detector, does not use Lowtran --> so far, ozone effect is missing
        if(!fMScattAnalysis && fDetAtGrnd)
	    TotTrans = fastc->Trans(*p,EUSO(),Trans);
	    
	// space detector case uses Lowtran --> ozone taken into account
	else if(!fMScattAnalysis && !fDetAtGrnd)
	    TotTrans = fLowToDetec->Trans(*p,EUSO(),Trans);
	    
	// MScatt analysis : does propagate photons, detector does not exist
	else {
	    TotTrans = 1;
	    Trans[0] = 1; Trans[1] = 1; Trans[2] = 1; Trans[3] = 1; 
	}
	p->SetLastTrans(TotTrans,"tot");
	p->SetLastTrans(Trans[1],"rayl");
	p->SetLastTrans(Trans[2],"ozone");
	p->SetLastTrans(Trans[3],"aero");
	p->SetLastTrans(TotTrans/Trans[0],"cloud");

        // tell if photon is absorbed
        if(TotTrans < rndm->Rndm())  p->SetAbsorbed();
	
        // Dump CPU commentaries
        fraction = 100*Float_t(nTracked)/Float_t(nTotal);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::ScanTempList() {
    //
    // remove unrelevant photons (status > 4)
    // if fKeepAll == false : call PropagateTempListToDetector() --> photons absorbed are not kept
    //
    // if GROUND detector mode : check geometrical cuts
    // check if photon is a candidate : if not, it is lost
    //
    
    SinglePhoton* p = 0;
    for(UInt_t m=0; m < fTempList.size(); m++) {
        p = fTempList[m];
        if(p->Status() > 4) {
	    SafeDelete(fTempList[m]);
	}
	else if(fDetAtGrnd && !IsSeenByGroundDetector(*p)) {
	    SafeDelete(fTempList[m]);
	}
    }
    
    if(!fKeepAll) PropagateTempListToDetector();
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::PropagateTempListToDetector() {
    //
    // same as PropagateToDetector but :
    // - act on fTempList
    // - disable fine transmission study
    //

    // init
    EarthVector dirtest(1);
    Double_t Trans[4];
    Double_t TotTrans(0.);
    TRandom* rndm = EsafRandom::Get();
    SinglePhoton* p = 0;
    size_t nTotal(0),nTracked(0);
    Float_t fraction(0); 
    Bool_t next(0),subnext(0);
    Float_t left(0),right(10),subleft(0),subright(2.5);
    nTotal = fTempList.size();
    Msg(EsafMsg::Info) << "Propagation TempList to Detector: 0%" << MsgFlush;
    
    static FastRadiativeProcessesCalculator* fastc = new FastRadiativeProcessesCalculator();

    // loop over the list of SinglePhoton
    for(UInt_t m=0; m < fTempList.size(); m++) {
        p = fTempList[m];
        nTracked++;
	if(!p) continue;
		
	// photon direction is toward EUSO
        p->SetDir((EUSO() - p->Pos()).Unit());

        // calculate transmission from photon position to EUSO
	// for ground detector, does not use Lowtran --> so far, ozone effect is missing
        if(!fMScattAnalysis && fDetAtGrnd) {
	    TotTrans = fastc->Trans(*p,EUSO(),Trans);
	}
	// space detector case uses Lowtran --> ozone taken into account
	else if(!fMScattAnalysis && !fDetAtGrnd) TotTrans = fLowToDetec->Trans(*p,EUSO(),Trans);
	// MScatt analysis : does propagate photons, detector does not exist
	else {
	    TotTrans = 1;
	    Trans[0] = 1; Trans[1] = 1; Trans[2] = 1; Trans[3] = 1; 
	}
	// fine transmission study is disabled
	p->SetLastTrans(-1,"tot");
	p->SetLastTrans(-1,"rayl");
	p->SetLastTrans(-1,"ozone");
	p->SetLastTrans(-1,"aero");
	p->SetLastTrans(-1,"cloud");

        // photon is transmitted or absorbed
        if(TotTrans < rndm->Rndm())  p->SetAbsorbed();
	
        if(p->IsAbsorbed()) SafeDelete(fTempList[m]);
	
	
        // Dump CPU commentaries
        fraction = 100*Float_t(nTracked)/Float_t(nTotal);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
}

//_______________________________________________________________________________________________________________________
Bool_t MCRadiativeTransfer::IsSeenByGroundDetector(const SinglePhoton& p) const {
    //
    // for ground detector only, assumed to be half a sphere
    // if dist(photon,detec) < fDistance_cut, photon is lost 
    // distribute photons on a sphere : if photons are on the bottom hemisphere, they are lost
    //

    // photons with pos.Z < 0 won't be seen by ground detector
    if(p.Pos().Z() < 0) return false;

    Double_t dist_to_detec = (EUSO() - p.Pos()).Mag();

    // check fDistance_cut
    if(dist_to_detec < fDistance_cut) return false;
    
    
    
    // distribute photon on a local sphere (radius = detector radius)
    // local frame is centered on sphere origin
    TRandom* rndm = EsafRandom::Get();
    Double_t Rsphere = EusoRadius();
    EarthVector pos_on_sphere(1);
    EarthVector pos_in_MESframe(1);
    EarthVector pos_local(0,0,dist_to_detec);  // photon position in local frame
    EarthVector dir_local(1);
    
      // position of photon on a horizontal disk cutting sphere in two hemisphere (<=> as sphere is seen from photon position)
      // this disk contains the local frame origin
    Double_t r(0.), phi(0.); // polar coordinates
    r = Rsphere * sqrt(rndm->Rndm());
    phi = TwoPi() * rndm->Rndm();
    EarthVector pos_on_plane(r*cos(phi),r*sin(phi),0.);
    dir_local = (pos_on_plane - pos_local).Unit();
    
      // then get photon position on the local sphere
    Double_t mag(0);
    Double_t b = pos_on_plane*dir_local;
    Double_t c = pos_on_plane.Mag2() - Rsphere*Rsphere;
    pair<Int_t,Double_t*>& solu = findRoots(1.,2*b,c);
    if(solu.first == 0) {
        Msg(EsafMsg::Warning) << "<IsSeenByGroundDetector> no impact on local sphere : SHOULD NOT OCCUR, photon is lost" << MsgDispatch;
        return false;
    }
    if(solu.first == 1) mag = solu.second[0];
    else if(solu.first ==2) mag = min(solu.second[0],solu.second[1]);
    if((mag - kTolerance) > 0) {
        Msg(EsafMsg::Warning) << "<IsSeenByGroundDetector> mag should always be < 0" << MsgDispatch;
        Msg(EsafMsg::Warning) << "<IsSeenByGroundDetector> HERE mag = " <<mag <<"   and it SHOUD NOT OCCUR" << MsgDispatch;
    }
    pos_on_sphere = pos_on_plane + mag*dir_local;

      // define rotation and apply it to pos_on_sphere in order to get photons position in MES
    EarthVector Uz(0,0,1);  // main axis of local sphere
    EarthVector axis = p.Pos().Cross(Uz);
    pos_in_MESframe = pos_on_sphere;
    pos_in_MESframe.Rotate(-p.Pos().Theta(),axis);
    
    
    // check if pos_in_MESframe is in the upper hemisphere
    if(pos_in_MESframe.Z() >= 0) return true;
    else return false;
}

//_______________________________________________________________________________________________________________________
void MCRadiativeTransfer::ConvertIntoPonP() {
    //
    // generate PhotonsOnPupil (fPhotons) from ListPhotonsInAtmosphere (fTotalList)
    // SinglePhoton are destroyed after their conversion into ParentPhoton
    //
    
    // initializations
    if(!fPhotons) fPhotons = new ListPhotonsOnPupil((vector<ParentPhoton*>*)NULL);
    if(!fPhotons) Msg(EsafMsg::Panic) << "MCRadiativeTransfer::PropagateAllToDetector, NULL fPhotons : Memory pb" << MsgDispatch;
    
    size_t nTotal(0),nTracked(0);
    TVector3 local_dir, pos;
    Float_t fraction(0); 
    Bool_t next(0),subnext(0);
    Float_t left(0),right(10),subleft(0),subright(2.5);
    nTotal = fTotalList->GetSingleEntries();
    SinglePhoton* p = 0;

    // build PhotonsOnPupil's frame
    BuildPupilFrame(fPhotons);
    Msg(EsafMsg::Info) << "Convert PhotonsInAtmosphere into PhotonsOnPupil: 0%" << MsgFlush;
    

    // convert photons
    fTotalList->ResetSingleCounter();   // for below extraction from list
    while(true) {
        p = fTotalList->GetSingle();
        if(!p) break;
        nTracked++;
	
	// Sample a random photon position on pupil
	pos =  p->Pos();
	local_dir = p->Dir();
	p->SetPosInAtmo(p->Pos());
	p->AddToPosTof(EUSO() - p->Pos());
	RamdomPosOnPupil(fPhotons,pos,local_dir);


	// check if photon is within the FoV
	if(!GetDetGeometry()->IsInFoV(local_dir)) p->SetOutFoV();
	else p->SetOutFoV(false);


	// if photon transmitted, becomes a photon on pupil
	// FoV 'status' not relevant here (too simply treated, ONLY a flag in RT part)
	// -->  will be considered in detector part
	if(!p->IsAbsorbed()) fPhotons->Add(*p,pos,local_dir);


        // Dump CPU commentaries
        fraction = 100*Float_t(nTracked)/Float_t(nTotal);
        if (left<fraction && fraction <=right) 
            next = kFALSE;
        else if (fraction>right) {
            next = kTRUE;
            left = right;
            right += 10;
        }
        if (subleft<fraction  && fraction <=subright) {
            subnext = kFALSE;
        }
        if (fraction > subright) {
            subnext = kTRUE;
            subleft = subright;
            subright +=2;
        }
        if (next) Msg(EsafMsg::Info) << left << "%" << MsgFlush;
        if (subnext) Msg(EsafMsg::Info)<< "." << MsgFlush;	 
    }
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
#ifdef DEBUG
    Msg(EsafMsg::Debug) << "Number of ParentPhoton = " << fPhotons->GetNphotons() << MsgDispatch;
#endif
}


